; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

disasmKeyTable:
	.db	skDown
	.dw	disasmNext
	.db	skUp
	.dw	disasmPrev
	.db	skRight
	.dw	disasmNext
	.db	skLeft
	.dw	disasmPrev
	.db	skDel
	.dw	Disassembler
	.db	skRecip
	.dw	HexEditor
	.db	skYEqu
	.dw	disasmCase
	.db	skWindow
	.dw	disasmBcalls
	.db	skTrace
	.dw	disasmPrevPage
	.db	skGraph
	.dw	disasmPage
	.db	skPower
	.dw	hexEditPage
	.db	skTan
	.dw	gotoAddress
;	.db	skAlpha
;	.dw	editSetOffsetReturn
	.db	sk0
	.dw	exchangeCurrentAddressWithBookmark0
	.db	sk1
	.dw	goBookmark1
	.db	sk2
	.dw	goBookmark2
	.db	sk3
	.dw	goBookmark3
	.db	sk4
	.dw	goBookmark4
	.db	sk5
	.dw	goBookmark5
	.db	sk6
	.dw	goBookmark6
	.db	sk7
	.dw	goBookmark7
	.db	sk8
	.dw	goBookmark8
	.db	sk9
	.dw	goBookmark9
	.db	skClear
	.dw	MainMenu
	.db	0

;====== Constants ==============================================================
; Instruction Bitfields
; 7  6  5  4  3  2  1  0
; X  X  Y  Y  Y  Z  Z  Z
;       P  P  Q
;       PP QQ QQ
argX		.equ	0
argY		.equ	1
argZ		.equ	2
argP		.equ	3
argQ		.equ	4
argPP		.equ	5
argQQ		.equ	6
; bit 0=signed, bit 1=compute actual address
isArgImm	.equ	6
isArgCached	.equ	3
isArg16		.equ	2
isArgRelAddr	.equ	1
isArgSigned	.equ	0
argCached	.equ	8 ; for index offset in autocopy
argImm16	.equ	128+64+4 ; bit 2: 16-bit, ignore other bits
argImm8		.equ	128+64+0 ; for 8-bit loads
argSImm8	.equ	128+64+2 ; for jr and djnz
argOffset	.equ	128+64+1 ; for index offsets

;
subLd		.equ	1
subHl		.equ	subLd+1
subBcInd	.equ	subHl+1
subDeInd	.equ	subBcInd+1
subRst		.equ	subDeInd+1
subJr		.equ	subRst+1
subJp		.equ	subJr+1
subAdd		.equ	subJp+1
subAdc		.equ	subAdd+1
subSub		.equ	subAdc+1
subSbc		.equ	subSub+1
subRet		.equ	subSbc+1
subEx		.equ	subRet+1
subCall		.equ	subEx+1
subBit		.equ	subCall+1
subRes		.equ	subBit+1
subSet		.equ	subRes+1
subIn		.equ	subSet+1
subOut		.equ	subIn+1
subInc		.equ	subOut+1
subDec		.equ	subInc+1
subCPort	.equ	subDec+1
subIm		.equ	subCPort+1
lastSub		.equ	subIm+1
;
specialJump	.equ	128
acceptIndexReg	.equ	64

; Decoding Branch
; Bit 7: Set = mask out bit 7 and use as string #
; Bit 6: Set = decode field & use as index for instr string #
; Bit 5: Set = decode field & use as index for jump to another decoding table
decodeAsString		.equ	128
decodeAndIndexForString	.equ	64
decodeAndIndexForJump	.equ	32


;====== ========================================================================
argDescTable:
; Bits 0-2: number of rotations
; Bits 3-5: mask
	.db	00011110b	; x
	.db	00111011b	; y
	.db	00111000b	; z
	.db	00011100b	; p
	.db	00001011b	; q
	.db	00001101b	; pp
	.db	00011011b	; qq


;====== Instruction Decoding Tables ============================================

primaryDecodingTable:
	.db	decodeAndIndexForJump+argX
_:	.db	prime0-$
	.db	prime1-$
	.db	prime2-$
	.db	prime3-$
prime0:
	.db	decodeAndIndexForJump+argZ
	.db	prime00-$
	.db	prime01-$
	.db	prime02-$
	.db	prime03-$
	.db	prime04-$
	.db	prime05-$
	.db	prime06-$
	.db	prime07-$
prime00:
	.db	decodeAndIndexForString+argY ; actually smaller than making another table
	.db	instrNop
	.db	instrExAF
	.db	instrDjnz
	.db	instrJr
	.db	instrJrCc
	.db	instrJrCc
	.db	instrJrCc
	.db	instrJrCc
prime01:
	.db	decodeAndIndexForString+argQ
	.db	instrLdRpImm16
	.db	instrAddHlRp
prime02:
	.db	decodeAndIndexForString+argY
	.db	instrLdBcA
	.db	instrLdABc
	.db	instrLdDeA
	.db	instrLdADe
	.db	instrLdImm16Hl
	.db	instrLdHlImm16
	.db	instrLdImm16A
	.db	instrLdAImm16
prime03:
	.db	decodeAndIndexForString+argQ
	.db	instrIncRP
	.db	instrDecRP
prime04:
	.db	decodeAsString+instrIncR
prime05:
	.db	decodeAsString+instrDecR
prime06:
	.db	decodeAsString+instrLdRImm8
prime07:
	.db	decodeAndIndexForString+argY
	.db	instrRlca
	.db	instrRrca
	.db	instrRla
	.db	instrRra
	.db	instrDaa
	.db	instrCpl
	.db	instrScf
	.db	instrCcf
prime1:
	.db	decodeAsString+instrLdRR
prime2:
	.db	decodeAsString+instrAluAR
prime3:
	.db	decodeAndIndexForJump+argZ
	.db	prime30-$
	.db	prime31-$
	.db	prime32-$
	.db	prime33-$
	.db	prime34-$
	.db	prime35-$
	.db	prime36-$
	.db	prime37-$
prime30:
	.db	decodeAsString+instrRetCc
prime31:
	.db	decodeAndIndexForString+argY
	.db	instrPop
	.db	instrRet
	.db	instrPop
	.db	instrExx
	.db	instrPop
	.db	instrJpHl
	.db	instrPop
	.db	instrLdSpHl
prime32:
	.db	decodeAsString+instrJpCc
prime33:
	.db	decodeAndIndexForString+argY
	.db	instrJp
	.db	instrCB
	.db	instrOut8
	.db	instrIn8
	.db	instrExSpHl
	.db	instrExDeHl
	.db	instrDi
	.db	instrEi
prime34:
	.db	decodeAsString+instrCallCc
prime35:
	.db	decodeAndIndexForString+argY
	.db	instrPush
	.db	instrCall
	.db	instrPush
	.db	instrDD
	.db	instrPush
	.db	instrED
	.db	instrPush
	.db	instrFD
prime36:
	.db	decodeAsString+instrAluImm8
prime37:
	.db	decodeAndIndexForString+argY
	.db	instrRst00h
	.db	instrRst08h
	.db	instrRst10h
	.db	instrRst18h
	.db	instrRst20h
	.db	instrRst28h
	.db	instrRst30h
	.db	instrRst38h

;------ CB Decoding Table ------------------------------------------------------
cbDecodingTable:
	.db	decodeAndIndexForString+argX
	.db	instrRot
	.db	instrBitYR
	.db	instrResYR
	.db	instrSetYR
cbAutocopyDecodingTable:
	.db	decodeAndIndexForJump+argZ
	.db	cba-$
	.db	cba-$
	.db	cba-$
	.db	cba-$
	.db	cba-$
	.db	cba-$
	.db	cbna-$
	.db	cba-$
cba:
	.db	decodeAndIndexForString+argX
	.db	instrRotAC
	.db	instrBitYRAC
	.db	instrResYRAC
	.db	instrSetYRAC
cbna:
	.db	decodeAndIndexForString+argX
	.db	instrRotNAC
	.db	instrBitYRNAC
	.db	instrResYRNAC
	.db	instrSetYRNAC

;------ ED Decoding Table ------------------------------------------------------
edDecodingTable:
	.db	decodeAndIndexForJump+argX
	.db	edInvalid-$
	.db	ed1-$
	.db	ed2-$
	.db	edInvalid-$
ed1:
	.db	decodeAndIndexForJump+argZ
	.db	ed10-$
	.db	ed11-$
	.db	ed12-$
	.db	ed13-$
	.db	ed14-$
	.db	ed15-$
	.db	ed16-$
	.db	ed17-$
ed10:
	.db	decodeAsString+instrInC
ed11:
	.db	decodeAsString+instrOutC
ed12:
	.db	decodeAndIndexForString+argQ
	.db	instrSbcHl
	.db	instrAdcHl
ed13:
	.db	decodeAndIndexForString+argQ
	.db	instrLdImm16IndRp
	.db	instrLdRpImm16Ind
ed14:
	.db	decodeAsString+instrNeg
ed15:
	.db	decodeAndIndexForString+argY
	.db	instrRetn
	.db	instrReti
	.db	instrInvalid
	.db	instrInvalid
	.db	instrInvalid
	.db	instrInvalid
	.db	instrInvalid
	.db	instrInvalid
ed16:
	.db	decodeAndIndexForString+argQQ
	.db	instrIm0
	.db	instrIm10
	.db	instrIm1
	.db	instrIm2
ed17:
	.db	decodeAndIndexForString+argY
	.db	instrLdIA
	.db	instrLdRA
	.db	instrLdAI
	.db	instrLdAR
	.db	instrRrd
	.db	instrRld
	.db	instrInvalid
	.db	instrInvalid
ed2:
	.db	decodeAndIndexForJump+argPP
	.db	edInvalid-$
	.db	ed2Cont-$
ed2Cont:
	.db	decodeAndIndexForJump+argZ
	.db	ed20-$
	.db	ed21-$
	.db	ed22-$
	.db	ed23-$
	.db	edInvalid-$
	.db	edInvalid-$
	.db	edInvalid-$
	.db	edInvalid-$
ed20:
	.db	decodeAsString+instrLdBlk
ed21:
	.db	decodeAsString+instrCpBlk
ed22:
	.db	decodeAsString+instrInBlk
ed23:
	.db	decodeAndIndexForString+argQQ
	.db	instrOutBlk
	.db	instrOutBlk
	.db	instrOtBlk
	.db	instrOtBlk
edInvalid:
	.db	decodeAsString+instrInvalid


;====== Argument Strings and Other Stuff =======================================
subTbl:
_:	.db	(+_)-(-_)-1, "LD "
_:	.db	+_--_-1, "HL"
_:	.db	+_ - -_ -1, "(BC)"
_:	.db	+_ - -_ -1, "(DE)"
_:	.db	+_ - -_ -1, "RST "
_:	.db	+_ - -_ -1, "JR "
_:	.db	+_ - -_ -1, "JP "
_:	.db	+_ - -_ -1, "ADD "
_:	.db	+_ - -_ -1, "ADC "
_:	.db	+_ - -_ -1, "SUB "
_:	.db	+_ - -_ -1, "SBC "
_:	.db	+_ - -_ -1, "RET "
_:	.db	+_ - -_ -1, "EX "
_:	.db	+_ - -_ -1, "CALL "
_:	.db	+_ - -_ -1, "BIT "
_:	.db	+_ - -_ -1, "RES "
_:	.db	+_ - -_ -1, "SET "
_:	.db	+_ - -_ -1, "IN "
_:	.db	+_ - -_ -1, "OUT "
_:	.db	+_ - -_ -1, "INC "
_:	.db	+_ - -_ -1, "DEC "
_:	.db	+_ - -_ -1, "(C)"
_:	.db	+_ - -_ -1, "IM "
indexArg:
_:	.db	(+_)-(-_)-1, "(IX", argOffset, ")"
;indeyArg:
;_:	.db	(+_)-(-_)-1, "(IY", argOffset, ")"
_:
indexX:
	.db	2, "IX"
;indexY:
;	.db	2, "IY"
argMetaTable:
	.dw	argTblRP
	.dw	argTblRP2
	.dw	argTblCC
	.dw	argTblBit
	.dw	argTblRot
	.dw	argTblAlu
	.dw	argTblBlockMode
	.dw	argTblR
argTblRP:
tblRP		.equ	0*8+128
	.db	2, "BC"
	.db	2, "DE"
	.db	2, "HL"
	.db	2, "SP"
argTblRP2:
tblRP2		.equ	1*8+128
	.db	2, "BC"
	.db	2, "DE"
	.db	2, "HL"
	.db	2, "AF"
argTblCC:
tblCC		.equ	2*8+128
	.db	2, "NZ"
	.db	1, "Z"
	.db	2, "NC"
	.db	1, "C"
	.db	2, "PO"
	.db	2, "PE"
	.db	1, "P"
	.db	1, "M"
argTblBit:
tblBit		.equ	3*8+128
	.db	1, "0"
	.db	1, "1"
	.db	1, "2"
	.db	1, "3"
	.db	1, "4"
	.db	1, "5"
	.db	1, "6"
	.db	1, "7"
argTblRot:
tblRot		.equ	4*8+128
	.db	4, "RLC "
	.db	4, "RRC "
	.db	3, "RL "
	.db	3, "RR "
	.db	4, "SLA "
	.db	4, "SRA "
	.db	4, "SLL "
	.db	4, "SRL "
argTblAlu:
tblAlu		.equ	5*8+128
	.db	6, "ADD A,"
	.db	6, "ADC A,"
	.db	4, "SUB "
	.db	6, "SBC A,"
	.db	4, "AND "
	.db	4, "XOR "
	.db	3, "OR "
	.db	3, "CP "
argTblBlockMode:
tblBlk		.equ	6*8+128
	.db	1, "I"
	.db	1, "D"
	.db	2, "IR"
	.db	2, "DR"
argTblR:
tblR		.equ	7*8+128
	.db	1, "B"
	.db	1, "C"
	.db	1, "D"
	.db	1, "E"
	.db	1, "H"
	.db	1, "L"
	.db	4, "(HL)"
	.db	1, "A"


;====== Instruction Strings ====================================================
instrNameTbl:
instrNoni	.equ	0
_:	.db	(+_)-(-_)-1
	.db	"NONI"
instrInvalid	.equ	instrNoni+1
_:	.db	(+_)-(-_)-1
	.db	"BADOP"
instrNop	.equ	instrInvalid+1
_:	.db	(+_)-(-_)-1
	.db	"NOP"
instrExAF	.equ	instrNop+1
_:	.db	(+_)-(-_)-1
	.db	subEx, "AF,AF", 27h
instrDjnz	.equ	instrExAF+1
_:	.db	(+_)-(-_)-1
	.db	"DJNZ ", argSImm8
instrJr	.equ	instrDjnz+1
_:	.db	(+_)-(-_)-1
	.db	subJr, argSImm8
instrJrCc	.equ	instrJr+1
_:	.db	(+_)-(-_)-1
	.db	subJr, tblCC+argQQ, ",", argSImm8
instrLdRpImm16	.equ	instrJrCc+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subLd, tblRP+argP, ",", argImm16
instrAddHlRp	.equ	instrLdRpImm16+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subAdd, subHl, ",", tblRP+argP
instrLdBcA	.equ	instrAddHlRp+1
_:	.db	(+_)-(-_)-1
	.db	subLd, subBcInd, ",A"
instrLdDeA	.equ	instrLdBcA+1
_:	.db	(+_)-(-_)-1 
	.db	subLd, subDeInd, ",A"
instrLdImm16Hl	.equ	instrLdDeA+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subLd, "(", argImm16, "),", subHl
instrLdImm16A	.equ	instrLdImm16Hl+1
_:	.db	(+_)-(-_)-1
	.db	subLd, "(", argImm16, "),A"
instrLdABc	.equ	instrLdImm16A+1
_:	.db	(+_)-(-_)-1
	.db	subLd, "A,", subBcInd
instrLdADe	.equ	instrLdABc+1
_:	.db	(+_)-(-_)-1
	.db	subLd, "A,", subDeInd
instrLdHlImm16	.equ	instrLdADe+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subLd, subHl, ",(", argImm16, ")"
instrLdAImm16	.equ	instrLdHlImm16+1
_:	.db	(+_)-(-_)-1
	.db	subLd, "A,(", argImm16, ")"
instrIncR	.equ	instrLdAImm16+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subInc, tblR+argY
instrDecR	.equ	instrIncR+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subDec, tblR+argY
instrIncRP	.equ	instrDecR+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subInc, tblRP+argP
instrDecRP	.equ	instrIncRP+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subDec, tblRP+argP
instrLdRImm8	.equ	instrDecRP+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subLd, tblR+argY, ",", argImm8
instrRlca	.equ	instrLdRImm8+1
_:	.db	(+_)-(-_)-1
	.db	"RLCA"
instrRrca	.equ	instrRlca+1
_:	.db	(+_)-(-_)-1
	.db	"RRCA"
instrRla	.equ	instrRrca+1
_:	.db	(+_)-(-_)-1
	.db	"RLA"
instrRra	.equ	instrRla+1
_:	.db	(+_)-(-_)-1
	.db	"RRA"
instrDaa	.equ	instrRra+1
_:	.db	(+_)-(-_)-1
	.db	"DAA"
instrCpl	.equ	instrDaa+1
_:	.db	(+_)-(-_)-1
	.db	"CPL"
instrScf	.equ	instrCpl+1
_:	.db	(+_)-(-_)-1
	.db	"SCF"
instrCcf	.equ	instrScf+1
_:	.db	(+_)-(-_)-1
	.db	"CCF"
instrLdRR	.equ	instrCcf+1
_:	.db	(+_)-(-_)-1+specialJump+acceptIndexReg
	.dw	checkHalt
instrLdRRReal	.equ	instrLdRR+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subLd, tblR+argY, ",", tblR+argZ
instrHalt	.equ	instrLdRRReal+1
_:	.db	(+_)-(-_)-1
	.db	"HALT"
instrAluAR	.equ	instrHalt+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	tblAlu+argY, tblR+argZ
instrRetCc	.equ	instrAluAR+1
_:	.db	(+_)-(-_)-1
	.db	subRet, tblCC+argY
instrPop	.equ	instrRetCc+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	"POP ", tblRP2+argP
instrRet	.equ	instrPop+1
_:	.db	(+_)-(-_)-1
	.db	subRet
instrExx	.equ	instrRet+1
_:	.db	(+_)-(-_)-1
	.db	"EXX"
instrJpHl	.equ	instrExx+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subJp, subHl
instrLdSpHl	.equ	instrJpHl+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subLd, "SP,", subHl
instrJpCc	.equ	instrLdSpHl+1
_:	.db	(+_)-(-_)-1
	.db	subJp, tblCC+argY, ",", argImm16
instrJp		.equ	instrJpCc+1
_:	.db	(+_)-(-_)-1
	.db	subJp, argImm16
instrCB		.equ	instrJp+1
_:	.db	(+_)-(-_)-1+specialJump+acceptIndexReg
	.dw	processPrefixCB
instrOut8	.equ	instrCB+1
_:	.db	(+_)-(-_)-1
	.db	subOut, "(", argImm8, "),A"
instrIn8	.equ	instrOut8+1
_:	.db	(+_)-(-_)-1
	.db	subIn, "A,(", argImm8, ")"
instrExSpHl	.equ	instrIn8+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subEx, "(SP),", subHl
instrExDeHl	.equ	instrExSpHl+1
_:	.db	(+_)-(-_)-1
	.db	subEx, "DE,", subHl
instrDi		.equ	instrExDeHl+1
_:	.db	(+_)-(-_)-1
	.db	"DI"
instrEi		.equ	instrDi+1
_:	.db	(+_)-(-_)-1
	.db	"EI"
instrCallCc	.equ	instrEi+1
_:	.db	(+_)-(-_)-1
	.db	subCall, tblCC+argY, ",", argImm16
instrPush	.equ	instrCallCc+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	"PUSH ", tblRP2+argP
instrCall	.equ	instrPush+1
_:	.db	(+_)-(-_)-1
	.db	subCall, argImm16
instrDD		.equ	instrCall+1
_:	.db	(+_)-(-_)-1+specialJump
	.dw	processPrefixDD
instrED		.equ	instrDD+1
_:	.db	(+_)-(-_)-1+specialJump
	.dw	processPrefixED
instrFD		.equ	instrED+1
_:	.db	(+_)-(-_)-1+specialJump
	.dw	processPrefixFD
instrAluImm8	.equ	instrFD+1
_:	.db	(+_)-(-_)-1
	.db	tblAlu+argY, argImm8
instrRst00h	.equ	instrAluImm8+1
_:	.db	(+_)-(-_)-1
	.db	subRst, "00h"
instrRst08h	.equ	instrRst00h+1
_:	.db	(+_)-(-_)-1
	.db	subRst, "08h"
instrRst10h	.equ	instrRst08h+1
_:	.db	(+_)-(-_)-1
	.db	subRst, "10h"
instrRst18h	.equ	instrRst10h+1
_:	.db	(+_)-(-_)-1
	.db	subRst, "18h"
instrRst20h	.equ	instrRst18h+1
_:	.db	(+_)-(-_)-1
	.db	subRst, "20h"
instrRst28h	.equ	instrRst20h+1
_:	.db	(+_)-(-_)-1+specialJump
	.dw	checkDoBcall
instrRst28hReal	.equ	instrRst28h+1
_:	.db	(+_)-(-_)-1
	.db	subRst, "28h"
instrRst30h	.equ	instrRst28hReal+1
_:	.db	(+_)-(-_)-1
	.db	subRst, "30h"
instrRst38h	.equ	instrRst30h+1
_:	.db	(+_)-(-_)-1
	.db	subRst, "38h"
instrBcall	.equ	instrRst38h+1
_:	.db	(+_)-(-_)-1
	.db	"B", subCall, argImm16

;------ Instruction CB Table ---------------------------------------------------
instrTableCB:
instrRot	.equ	instrBcall+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	tblRot+argY, tblR+argZ
instrBitYR	.equ	instrRot+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subBit, tblBit+argY, ",", tblR+argZ
instrResYR	.equ	instrBitYR+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subRes, tblBit+argY, ",", tblR+argZ
instrSetYR	.equ	instrResYR+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subSet, tblBit+argY, ",", tblR+argZ
instrRotAC	.equ	instrSetYR+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	tblRot+argY, tblR+argZ, ",(IX", argOffset+argCached, ")"
instrBitYRAC	.equ	instrRotAC+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
;	.db	"BIT ", tblBit+argY, ",", tblR+argZ, ",(IX", argOffset+argCached, ")"
	.db	subBit, tblBit+argY, ",", tblR+argZ
instrResYRAC	.equ	instrBitYRAC+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subRes, tblBit+argY, ",", tblR+argZ, ",(IX", argOffset+argCached, ")"
instrSetYRAC	.equ	instrResYRAC+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subSet, tblBit+argY, ",", tblR+argZ, ",(IX", argOffset+argCached, ")"
instrRotNAC	.equ	instrSetYRAC+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	tblRot+argY, "(IX", argOffset+argCached, ")"
instrBitYRNAC	.equ	instrRotNAC+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
;	.db	"BIT ", tblBit+argY, ",", tblR+argZ, ",(IX", argOffset+argCached, ")"
	.db	subBit, tblBit+argY, ",(IX", argOffset+argCached, ")"
instrResYRNAC	.equ	instrBitYRNAC+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subRes, tblBit+argY, ",(IX", argOffset+argCached, ")"
instrSetYRNAC	.equ	instrResYRNAC+1
_:	.db	(+_)-(-_)-1+acceptIndexReg
	.db	subSet, tblBit+argY, ",(IX", argOffset+argCached, ")"


;------ Instruction ED Table ---------------------------------------------------
instrTableED:
instrInC	.equ	instrSetYRNAC+1
_:	.db	(+_)-(-_)-1+specialJump
	.dw	checkIn
instrInCReal	.equ	instrInC+1
_:	.db	(+_)-(-_)-1
	.db	subIn, tblR+argY, ",", subCPort
instrInFC	.equ	instrInCReal+1
_:	.db	(+_)-(-_)-1
	.db	subIn, "F,", subCPort
instrOutC	.equ	instrInFC+1
_:	.db	(+_)-(-_)-1+specialJump
	.dw	checkOut
instrOutCReal	.equ	instrOutC+1
_:	.db	(+_)-(-_)-1
	.db	subOut, subCPort, ",", tblR+argY
instrOutC0	.equ	instrOutCReal+1
_:	.db	(+_)-(-_)-1
	.db	subOut, subCPort, ",0"
instrSbcHl	.equ	instrOutC0+1
_:	.db	(+_)-(-_)-1 ; There is no IX/IY variant for this instruction
	.db	subSbc, subHl, ",", tblRP+argP
instrAdcHl	.equ	instrSbcHl+1
_:	.db	(+_)-(-_)-1 ; There is no IX/IY variant for this instruction
	.db	subAdc, subHl, ",", tblRP+argP
instrLdImm16IndRp .equ	instrAdcHl+1
_:	.db	(+_)-(-_)-1
	.db	subLd, "(", argImm16, "),", tblRP+argP
instrLdRpImm16Ind .equ	instrLdImm16IndRp+1
_:	.db	(+_)-(-_)-1
	.db	subLd, tblRP+argP, ",(", argImm16, ")"
instrNeg	.equ	instrLdRpImm16Ind+1
_:	.db	(+_)-(-_)-1
	.db	"NEG"
instrRetn	.equ	instrNeg+1
_:	.db	(+_)-(-_)-1
	.db	"RETN"
instrReti	.equ	instrRetn+1
_:	.db	(+_)-(-_)-1
	.db	"RETI"
instrIm0	.equ	instrReti+1
_:	.db	(+_)-(-_)-1
	.db	subIm, "0"
instrIm10	.equ	instrIm0+1
_:	.db	(+_)-(-_)-1
	.db	subIm, "1/0"
instrIm1	.equ	instrIm10+1
_:	.db	(+_)-(-_)-1
	.db	subIm, "1"
instrIm2	.equ	instrIm1+1
_:	.db	(+_)-(-_)-1
	.db	subIm, "2"
instrLdIA	.equ	instrIm2+1
_:	.db	(+_)-(-_)-1
	.db	subLd, "I,A"
instrLdRA	.equ	instrLdIA+1
_:	.db	(+_)-(-_)-1
	.db	subLd, "R,A"
instrLdAI	.equ	instrLdRA+1
_:	.db	(+_)-(-_)-1
	.db	subLd, "A,I"
instrLdAR	.equ	instrLdAI+1
_:	.db	(+_)-(-_)-1
	.db	subLd, "A,R"
instrRrd	.equ	instrLdAR+1
_:	.db	(+_)-(-_)-1
	.db	"RRD"
instrRld	.equ	instrRrd+1
_:	.db	(+_)-(-_)-1
	.db	"RLD"
instrLdBlk	.equ	instrRld+1
_:	.db	(+_)-(-_)-1
	.db	"LD", tblBlk+argQQ
instrCpBlk	.equ	instrLdBlk+1
_:	.db	(+_)-(-_)-1
	.db	"CP", tblBlk+argQQ
instrInBlk	.equ	instrCpBlk+1
_:	.db	(+_)-(-_)-1
	.db	"IN", tblBlk+argQQ
instrOutBlk	.equ	instrInBlk+1
_:	.db	(+_)-(-_)-1
	.db	"OUT", tblBlk+argQQ
instrOtBlk	.equ	instrOutBlk+1
_:	.db	(+_)-(-_)-1
	.db	"OT", tblBlk+argQQ
_:
last_instr_string	.equ	instrOtBlk
