; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

; This file contains offsets of memory locations and constants that replace
; ti83plus.inc or ti84pcse.inc .

#define equ	.equ

;------ Macros -----------------------------------------------------------------
#macro osf
; Open Stack Frame
; Opens a stack frame.
	push	ix
	ld	ix, 0
	add	ix, sp
#endmacro

#macro csf
; Close Stack Frame
	pop	ix
#endmacro

#macro ldHlIndirect
	ld	a, (hl)
	inc	hl
	ld	h, (hl)
	ld	l, a
#endmacro

#macro cpHlDe
	or	a
	sbc	hl,de
	add	hl,de
#endmacro

#macro getLcdRegHl(regnum)
	ld a, regnum
	out (pLcdCmd), a
	out (pLcdCmd), a
	in a, (pLcdData)
	ld h, a
	in a, (pLcdData)
	ld l, a
#endmacro

#macro setLcdRegHl(regnum)
	ld a, regnum
	out (pLcdCmd), a
	out (pLcdCmd), a
	ld a, h
	out (pLcdData), a
	ld a, l
	out (pLcdData), a
#endmacro

#macro setLcdReg(regnum, highbyte, lowbyte)
	ld a, regnum
	out (pLcdCmd), a
	out (pLcdCmd), a
	ld a, highbyte
	out (pLcdData), a
	ld a, lowbyte
	out (pLcdData), a
#endmacro

#macro setLcdDReg(regnum, double)
	ld a, regnum
	out (pLcdCmd), a
	out (pLcdCmd), a
	ld a, lcdHigh(double)
	out (pLcdData), a
	ld a, lcdLow(double)
	out (pLcdData), a
#endmacro


;------ Structs ----------------------------------------------------------------
; 32-bit page/address pointer
ptrAddr		.equ	0
ptrPage		.equ	2
ptrHPage	.equ	3
#macro	ldDehlFromHl
	ld	e, (hl)
	inc	hl
	ld	d, (hl)
	inc	hl
	ld	a, (hl)
	inc	hl
	ld	h, (hl)
	ld	l, a
	ex	de, hl
#endmacro

#macro ldDehlFrom(location)
	ld	hl, (location)
	ld	de, (location+2)
#endmacro

#macro ldDehlToBc
	ld	a, l
	ld	(bc), a
	inc	bc
	ld	a, h
	ld	(bc), a
	inc	bc
	ld	a, e
	ld	(bc), a
	inc	bc
	ld	a, d
	ld	(bc), a
	inc	bc
#endmacro

#macro ldDehlTo(location)
	ld	(location), hl
	ld	(location+2), de
#endmacro


;------ Main Memory Layout Settings --------------------------------------------
start_of_static_vars	.equ	0FF00h
mflags			.equ	0FE80h
; FE01 is not free! It's the last byte of the IVT.
;flags		.equ	mflags
topOfStack	.equ	0FCF0h
stringFindFreeRAM .equ	0FC00h
IvtLocation	.equ	0FDh ; vector table at FD00h
IsrLocation	.equ	0FCh ; ISR at FCFCh
; Memory mapping
swapPort		.equ	6
swapPortHigh		.equ	0Eh
swapLoc			.equ	4000h
swapTop			.equ	8000h
; I don't know where else to put this.
ramPage0		equ	80h
ramPage1		equ	81h
ramPage2		equ	82h
ramPage3		equ	83h
ramPage4		equ	84h
ramPage5		equ	85h
ramPage6		equ	86h
ramPage7		equ	87h


;------ Character Constants ----------------------------------------------------
chQuotes	.equ	22h
chNewLine	.equ	01h
chEnter		.equ	01h
chTab		.equ	02h
chBackspace	.equ	03h
chDel		.equ	03h
ch1stPrintableChar	.equ	4
chHeartSuite	.equ	04h
chClear		.equ	04h
chDiamondSuite	.equ	05h
ch2ndClear	.equ	05h
chClubSuite	.equ	06h
chAlphaClear	.equ	06h
chSpadeSuite	.equ	07h
ch2ndDel	.equ	07h
chBullet	.equ	08h
chInsert	.equ	09h
chWhiteBullet	.equ	09h
chAlphaDel	.equ	0Ah
chRight		.equ	0Ah
chLeft		.equ	0Bh
chLeftRight	.equ	0Ch
chDown		.equ	0Dh
chUp		.equ	0Eh
chUpDown	.equ	0Fh
chBox		.equ	10h
chLightShade	.equ	11h
chMode		.equ	11h
chMediumShade	.equ	12h
ch2ndMode	.equ	12h
chDarkShade	.equ	13h
chAlphaMode	.equ	13h
chFilledShade	.equ	14h
ch2ndRight	.equ	15h
ch2ndLeft	.equ	16h
ch2ndEnter	.equ	17h
ch2ndDown	.equ	18h
ch2ndUp		.equ	19h
chAlphaRight	.equ	1Ah
chAlphaLeft	.equ	1Bh
chAlphaEnter	.equ	1Ch
chAlphaDown	.equ	1Dh
chAlphaUp	.equ	1Eh
chBeta		.equ	7Fh
chCur		.equ	80h
chCur2nd	.equ	81h
chCurAlpha	.equ	82h
chCurAlphaLwr	.equ	83h
chCurIns	.equ	84h
chCurIns2nd	.equ	85h
chCurInsAlpha	.equ	86h
chCurInsAlphaLwr	.equ	87h
chCurFull	.equ	88h

charWidth = 9
charHeight = 14
charLength = 16
textRows = 17
textCols = 32
colorScrnHeight = 240
colorScrnWidth = 320


;------ Settings ---------------------------------------------------------------
; Timers
cursorTimerDefaultValue .equ	512	; 0.5 seconds
suspendDelay	.equ	600	; 300 seconds
kbdScanDivisor		.equ	10	; The below values are multiples of 10.24 ms
minKeyHoldTime		.equ	3	; about 30 ms
keyBlockTime		.equ	2	; about 20 ms
keyFirstRepeatWaitTime	.equ	75	; about 750 ms
keyRepeatWaitTime	.equ	20	; about 200 ms
numOfBookmarks	.equ	9


;------ Flags ------------------------------------------------------------------
; Flags
appFlags	.equ	0h	;application flags
shiftFlags	.equ	0h	;[2nd] and [ALPHA] flags
textFlags	.equ	1h	;Text output flags
curFlags	.equ	1h	;Cursor
inputerFlags	.equ	2
instrFlags	.equ	3
editorFlags	.equ	4
;prgmFlags	.equ	4
findCounter	.equ	5

; shiftFlags
shift2nd	.equ	0	;1=[2nd] has been pressed
shift2ndMask	.equ	01h
shiftAlpha	.equ	1	;1=[ALPHA] has been pressed
shiftAlphaMask	.equ	02h
shiftLwrAlpha	.equ	2	;1=lower case, 0=upper case
shiftLwrAlphaMask	.equ	04h
shiftInsert	.equ	3
shiftInsertMask	.equ	08h
shiftALock	.equ	4	;1=alpha lock has been pressed
shiftALockMask	.equ	10h
shiftOther	.equ	5
shiftOtherMask	.equ	20h
shiftFull	.equ	0
shiftFullMask	.equ	01h
shiftBox	.equ	1
shiftBoxMask	.equ	02h
; appFlags
apdEnabled	.equ	6
apdEnabledMask	.equ	40h
apdNow		.equ	7
apdNowMask	.equ	80h
; textFlags
textInverse	.equ	0	;1=display inverse bit-map
textInverseMask	.equ	1
textAutoScroll	.equ	1	;1=auto-scroll text on last line
textAutoScrollMask	.equ	2
textScrolled	.equ	2	;1=screen scrolled
textNewlineClr	.equ	3
; curFlags
curAble 	.equ	4	;1=cursor flash is enabled
curAbleMask	.equ	10h
curOn		.equ	5	;1=cursor is showing
curOnMask	.equ	20h
;curLock 	.equ	5	;1=cursor is locked off
;curLockMask	.equ	20h
curFlash	.equ	6
curFlashMask	.equ	40h
; instrFlags ; These flags are used by the disassembler
insertStrSub	.equ	7	; in editorFlags
noBcall		.equ	6
noBcallMask	.equ	40h
isNeg		.equ	5
prefixCB	.equ	4
prefixED	.equ	3
subHLInd	.equ	2
prefixIndex	.equ	1
prefixIY	.equ	0	; Do not change
; Hex Editor flags
hexTexsMode		.equ	1
retDisasm		.equ	2
retDisasmMask		.equ	04h
disasmLowerCase		.equ	3
disasmLowerCaseMask	.equ	08h
findManLen		.equ	4
findUseWildcard		.equ	5
findQuick16		.equ	6
; inputerFlags
allowAlpha	.equ	0
allowLwrAlpha	.equ	1
allowNumbers	.equ	2
allowSymbols	.equ	3
allowControl	.equ	4
secondNibble	.equ	7


;------ Vars -------------------------------------------------------------------

;start_of_static_vars .equ saveSScreen
; General Variables
; start_of_static_vars is defined in the equates_bw or equates_pcse
quitaddr	.equ	0C000h
spaddr		.equ	quitaddr+2
pageAOrig	.equ	spaddr+2
pageBOrig	.equ	pageAOrig+2

; Cursor stuff
cursorChar	.equ	start_of_static_vars
cursorTimerPeriod	.equ	cursorChar+1
cursorTimer	.equ	cursorTimerPeriod+2

; APD
suspendTimer	.equ	cursorTimer+2

; Keyboard
kbdScanTimer		.equ	suspendTimer+2
lastKey			.equ	kbdScanTimer+1		; Normally 0
lastKeyHoldTimer	.equ	lastKey+1		; Normally minKeyHoldTime
lastKeyBlockTimer	.equ	lastKeyHoldTimer+1	; Normally keyBlockTime
lastKeyFirstRepeatTimer	.equ	lastKeyBlockTimer+1	; Normally keyFirstRepeatWaitTime
lastKeyRepeatTimer	.equ	lastKeyFirstRepeatTimer+1	; Normally keyRepeatWaitTime
keyBuffer		.equ	lastKeyRepeatTimer+1	; Normally 0

; Screen
currentRow	.equ	lastKeyRepeatTimer+2
currentCol	.equ	currentRow+1
tabPos1		.equ	currentCol+1
tabPos2		.equ	tabPos1+1
tabPos3		.equ	tabPos2+1
tabPos4		.equ	tabPos3+1
windTop		.equ	tabPos4+1
windLeft	.equ	windTop+1
windBottom	.equ	windLeft+1
windRight	.equ	windBottom+1

; Archive Parser
flashReadBuffer	.equ	windRight+1
flashReadBufferSize	.equ	16

; Port Monitor
portMonPort	.equ	flashReadBuffer+flashReadBufferSize
lcdPortMonPort	.equ	portMonPort+1

; Hex Editor
; This feature is slightly complicated.
bkmAddr			.equ	0
bkmPage			.equ	2
bkmHPage		.equ	3
bkmX			.equ	4
bkmY			.equ	5
sizeOfBkm		.equ	6
editFindSource		.equ	lcdPortMonPort+1
editFindPage		.equ	editFindSource+2
editFindHPage		.equ	editFindPage+1
editFindLength		.equ	editFindHPage+1
editWildcard		.equ	editFindLength+1
editQuickVal		.equ	editWildcard+1
editOffset		.equ	editQuickVal+2	; physical+offset=logical; physical=logical-offset
editLocation		.equ	editOffset+2
editPage		.equ	editLocation+2
editHPage		.equ	editPage+1
editXLoc		.equ	editHPage+1
editYLoc		.equ	editXLoc+1
chartY			.equ	editYLoc+1;chartIndex+1
chartX			.equ	chartY+1
editBookmarks		.equ	chartX+1
editBookmark0		.equ	editBookmarks+(sizeOfBkm*numOfBookmarks)
endOfHexEditorRam	.equ	editBookmark0+sizeOfBkm

; Disassembler
disasmStart	.equ	endOfHexEditorRam
disasmStartPage	.equ	disasmStart+2
disasmPtr	.equ	disasmStart+3
disasmPtrPage	.equ	disasmPtr+2
disasmPtrHPage	.equ	disasmPtr+3
instrTextBuf	.equ	disasmPtr+4
instrTextBufLen	.equ	16
instrTextBufText .equ	instrTextBuf+1
argBuf		.equ	instrTextBuf+instrTextBufLen+1
argBufLen	.equ	8
argBufText	.equ	argBuf+1
instrCode	.equ	argBuf+argBufLen+1
instrImm	.equ	instrCode+1
instrOffset	.equ	instrImm+2
;instrProcAddr	.equ	instrOffset+1
end_of_static_vars .equ instrOffset+1
size_of_static_vars .equ (end_of_static_vars) - (start_of_static_vars)


;====== Scan Code Equates ======================================================
skDown			.equ 01h
skLeft			.equ 02h
skRight			.equ 03h
skUp			.equ 04h
skEnter			.equ 09h
skAdd			.equ 0Ah
skSub			.equ 0Bh
skMul			.equ 0Ch
skDiv			.equ 0Dh
skPower			.equ 0Eh
skClear			.equ 0Fh
skChs			.equ 11h
sk3			.equ 12h
sk6			.equ 13h
sk9			.equ 14h
skRParen		.equ 15h
skTan			.equ 16h
skVars			.equ 17h
skDecPnt		.equ 19h
sk2			.equ 1Ah
sk5			.equ 1Bh
sk8			.equ 1Ch
skLParen		.equ 1Dh
skCos			.equ 1Eh
skPrgm			.equ 1Fh
skStat			.equ 20h
sk0			.equ 21h
sk1			.equ 22h
sk4			.equ 23h
sk7			.equ 24h
skComma			.equ 25h
skSin			.equ 26h
skMatrix		.equ 27h
skGraphvar		.equ 28h
skStore			.equ 2Ah
skLn			.equ 2Bh
skLog			.equ 2Ch
skSquare		.equ 2Dh
skRecip			.equ 2Eh
skMath			.equ 2Fh
skAlpha			.equ 30h
skGraph			.equ 31h
skTrace			.equ 32h
skZoom			.equ 33h
skWindow		.equ 34h
skYEqu			.equ 35h
sk2nd			.equ 36h
skMode			.equ 37h
skDel			.equ 38h


;====== Hardware Ports =========================================================
; p = Port
; LnkAst = Link Assist, Pg = Page
; Se = Silver Edition, Be = Basic Edition (anything NOT a TI-83+)
; Cfg = Configuration, Ctrl = Control, Buf = Buffer, Prot = Protection
; M = Memory, Int = Interrupt, Id = Identification, Crstl = Crystal
; Ack = Acknowledge
; Tmr = Timer, Div = Divided by, Rtc = Real Time Clock, Usb = USB
; Unk = Unknown, Exec = Execution
;
; R:n = RAM page n (always indexed from 0; no more saying RAM page 80h)
; F:nn = Flash page nn
; P:nn = Port nn
; LR:nn = LCD Register nn
pLink			equ	0
pKey			equ	1
pStatus			equ	2
pIntAck			equ	2
pIntMask		equ	3
pIntId			equ	4
pMMapTmrFreq		equ	4
pMPgC			equ	5
pLnkAstBeRead		equ	5
pMPgA			equ	6
pMPgB			equ	7
pLnkAstSeEnable		equ	8
pLnkAstSeStatus		equ	9
pLnkAstSeClkCpu6MHz	equ	9
pLnkAstSeInBuf		equ	0Ah
pLnkAstSeClkCpu15MHz	equ	0Ah
pLnkAstSeClkCpu20Mhz	equ	0Bh
pLnkAstSeClkCpu25Mhz	equ	0Ch
pLnkAstSeOutBuf		equ	0Dh
pMPgAHigh		equ	0Eh
pMPgBHigh		equ	0Fh
pLcdCmd			equ	10h
pLcdData		equ	11h
pProtCtrl		equ	14h
pAsicId			equ	15h
pBeFlashExecCtrl	equ	16h
pMd5RegA		equ	18h
pMd5RegB		equ	19h
pMd5RegC		equ	1Ah
pMd5RegD		equ	1Bh
pMd5RegX		equ	1Ch
pMd5Byte0		equ	1Ch
pMd5RegAC		equ	1Dh
pMd5Byte1		equ	1Dh
pMd5RegS		equ	1Eh
pMd5Byte2		equ	1Eh
pMd5FunctionSelect	equ	1Fh
pMd5Byte3		equ	1Fh
pCpuSpeed		equ	20h
pFlashRamType		equ	21h
pFlashExecLowerLimit	equ	22h
pFlashExecUpperLimit	equ	23h
pFlashExecHighBit	equ	24h
pRamExecLowerLimit	equ	25h
pRamExecUpperLimit	equ	26h
pBlockMRemapC		equ	27h
pBlockMRemapB		equ	28h
pLcdDelayCpu6MHz	equ	29h
pLcdDelayCpu15MHz	equ	2Ah
pLcdDelayCpu20MHz	equ	2Bh
pLcdDelayCpu25MHz	equ	2Ch
pCrstlKeepAlive		equ	2Dh
pMemDelay		equ	2Eh
pLcdGenDelay		equ	2Fh
pCrstlTmr1Freq		equ	30h
pCrstlTmr1Cfg		equ	31h
pCrstlTmr1Count		equ	32h
pCrstlTmr2Freq		equ	33h
pCrstlTmr2Cfg		equ	34h
pCrstlTmr2Count		equ	35h
pCrstlTmr3Freq		equ	36h
pCrstlTmr3Cfg		equ	37h
pCrstlTmr3Count		equ	38h
pGpioDirection		equ	39h
pGpioData		equ	3Ah
pRtcCtrl		equ	40h
pRtcSet0		equ	41h
pRtcSet1		equ	42h
pRtcSet2		equ	43h
pRtcSet3		equ	44h
pRtc0			equ	45h ; Reading
pRtc1			equ	46h
pRtc2			equ	47h
pRtc3			equ	48h

; Constants for bitfields
; St = Status, Batt = Batteries
; Rcv = Receive, Snd = Send, Err = Error
; B = "Bit"
; Port 2 (pStatus)
stBattGood		equ	1
stBattGoodB		equ	0
stLcdReady		equ	2
stLcdReadyB		equ	1
stFlashUnlocked		equ	4
stFlashUnlockedB	equ	3
;stIsNot83pSe		equ	5
stIsSeModel		equ	128
stIsSeModelB		equ	7
; Port 2 (pIntAck) Writes
intAckAll		equ	0
; Port 3 (pIntMask)
intOnKey		equ	1
intOnKeyB		equ	0
intTmr1			equ	2
intTmr1B		equ	1
intTmr2			equ	4
intTmr2B		equ	2
intDisableLowPowerMode	equ	8
intDisableLowPowerModeB	equ	3
intLinkport		equ	16
intLinkportB		equ	4
intMaskNormal		equ	intOnKey+intTmr1+intDisableLowPowerMode
; Port 4 (pIntId) Reads
intOnKeyDown		equ	8
intCrstlTmr1		equ	32
intCrstlTmr1B		equ	5
intCrstlTmr2		equ	64
intCrstlTmr2B		equ	6
intCrstlTmr3		equ	128
intCrstlTmr3B		equ	7
; Port 4 (pMMapTmrFreq) Writes
memMapMode0		equ	0
memMapMode1		equ	1
tmrFreq0		equ	0
tmrFreq1		equ	2
tmrFreq2		equ	4
tmrFreq3		equ	6
battVoltage0		equ	0
battVoltage1		equ	64
battVoltage2		equ	128
battVoltage3		equ	192
; Port 8 (pLnkAstSeEnable)
lnkSeRcv		equ	1
lnkSeRcvB		equ	0
lnkSeSnd		equ	2
lnkSeSndB		equ	1
lnkSeErr		equ	4
lnkSeErrB		equ	2
lnkSeTimeoutDisable	equ	64
lnkSeTimeoutDisableB	equ	6
lnkSeForceReset		equ	128
lnkSeForceResetB	equ	7
; Port 9 (pLnkAstSeStatus)
lnkSeRcving		equ	8
lnkSeRcvingB		equ	3
lnkSeRcvd		equ	16
lnkSeRcvdB		equ	4
lnkSeSndReady		equ	32
lnkSeSndReadyB		equ	5
lnkSeSndErr		equ	64
lnkSeSndErrB		equ	6
lnkSeSnding		equ	128
lnkSeSndingB		equ	7
lnkSeIntDisable		equ	128
; Ports 9-C (pLnkAstSeClkCpu6-25MHZ)
lnkSeBitHoldTimeMask	equ	31
lnkSeClockDiv1		equ	0
lnkSeClockDiv2		equ	32
lnkSeClockDiv4		equ	64
lnkSeClockDiv8		equ	96
lnkSeClockDiv16		equ	128
lnkSeClockDiv32		equ	160
lnkSeClockDiv64		equ	192
lnkSeStopClock		equ	224
; Port 14h (pProtCtrl)
protLockFlash		equ	0
protUnlockFlash		equ	1
; Port 15h (pAsicId)
asic83pTA2		equ	33h
asic84pTA2		equ	44h
asic84pTA3		equ	45h
asic84pTA1		equ	55h
; Port 1Fh (pMd5FunctionSelect)
md5FunctionFF		equ	0
md5FunctionGG		equ	1
md5FunctionHH		equ	2
md5FunctionII		equ	3
; Port 20h (pCpuSpeed)
cpu6MHz			equ	0
cpu15MHz		equ	1
cpu20MHz		equ	2	; left over from TI-83+SE beta
cpu25MHz		equ	3	; disabled on production models
; Port 21h (pFlashRamType)
flashTypeMask		equ	3
flashType1MB		equ	0
flashType2MB		equ	1
flashType4MB		equ	2
flashTyle8MB		equ	3
ramTypeMask		equ	30h
ramType32K		equ	00h
ramType64K		equ	10h
ramType128K		equ	20h
ramType256K		equ	30h
; Port 24h (pFlashExecHighBit)
flashExecLowerLimitHigh	equ	1
flashExecUpperLimitHigh	equ	2
; Ports 29h-2Ch (pLcdDelay6-25MHz)
delayFlashEnable	equ	1
delayRamEnable		equ	2
delayCycleCountMask	equ	0FCh
delayCycleCountShift	equ	2
delayCycleMin		equ	0Ch
; Port 2D (pCrstlKeepAlive)
crstlTmrKeepAlive	equ	1
crstlClockKeepAlive	equ	2
; Port 2E (pMemDelay)
delayFlashExec		equ	1
delayFlashRead		equ	2
delayFlashWrite		equ	4
delayRamExec		equ	10h
delayRamRead		equ	20h
delayRamWrite		equ	40h
; Port 2F (pLcdGenDelay)
cpu1LcdDelay48		equ	0
cpu1LcdDelay112		equ	1
cpu1LcdDelay176		equ	2
cpu1LcdDelay240		equ	3
cpu2LcdDelay48		equ	0
cpu2LcdDelay112		equ	4
cpu2LcdDelay176		equ	8
cpu2LcdDelay240		equ	12
cpu2LcdDelay304		equ	16
cpu2LcdDelay368		equ	20
cpu2LcdDelay432		equ	24
cpu2LcdDelay496		equ	28
cpu3LcdDelay48		equ	0
cpu3LcdDelay112		equ	32
cpu3LcdDelay176		equ	64
cpu3LcdDelay240		equ	96
cpu3LcdDelay304		equ	128
cpu3LcdDelay368		equ	160
cpu3LcdDelay432		equ	192
cpu3LcdDelay496		equ	224
; Crystal Timer Equates
tmrOff			equ	0
tmr11000Hz		equ	40h
tmr1000Hz		equ	41h
tmr100Hz		equ	42h
tmr10Hz			equ	43h
tmr32768Hz		equ	44h
tmr2048Hz		equ	45h
tmr128Hz		equ	46h
tmr8Hz			equ	47h
tmrCpu			equ	80h
tmrCpuDiv2		equ	81h
tmrCpuDiv4		equ	82h
tmrCpuDiv8		equ	84h
tmrCpuDiv16		equ	88h
tmrCpuDiv32		equ	90h
tmrCpuDiv64		equ	0A0h
tmrNullAck		equ	0
tmrLoop			equ	1
tmrInt			equ	2
tmrLoopAndInt		equ	3
; Port 40h (pRtcCtrl)
rtcClear		equ	1
rtcLoadCommand		equ	2

;------ Color LCD Control Registers --------------------------------------------
; Color LCD-Related Stuff
; lr = LCD Register, Disp = Display, Win = Window, Img = Image
; Col = Column, Cur = Cursor
; Use these macros with the bitfield definitions below.
#define lcdHigh(x) x >> 80
#define lcdLow(x) x & 255
lcdWidth = 320
lcdHeight = 240

; Registers
lrSync			equ	0
lrDriverCode		equ	0
lrOutCtrl1		equ	1
lrDriveCtrl		equ	2
lrEntryMode		equ	3
lrRow			equ	20h
lrCol			equ	21h
lrGram			equ	22h
lrWinTop		equ	50h
lrWinBottom		equ	51h
lrWinLeft		equ	52h
lrWinRight		equ	53h
lrBaseImgDispCtrl	equ	61h
lrHorizScroll		equ	6Ah

; Register 1 (lrOutCtrl1) Bitfields
lcdReverseRow		equ	0100h
lcdInterlaced		equ	0400h
; Register 2 (lrDriveCtrl) Bitfields
lcdEOR			equ	0100h
lcdBDivC		equ	0200h
; Register 3 (lrEntryMode) Bitfields
lcdAM			equ	0008h	; If set, the cursor moves left/right after every write.
lcdCurMoveHoriz		equ	lcdAM	; Alias
lcdRowInc		equ	0010h	; If unset, the cursor is instead decremented
lcdColInc		equ	0020h	
lcdORG			equ	0080h	; If set, when window is changed, cursor is reset to corner
lcdBGR			equ	1000h
lcdTRI			equ	4000h	; If set, 18-bit color is accepted
lcdDFM			equ	8000h	; Unpacked mode: Write R, G, and B separately if TRI is set
lcdDispCtrlDefault	equ	1038h
; Register 61 (lrBaseImgDispCtrl) Bitfields
lcdREV			equ	0001h	; If set, reverse color mode
lcdVLE			equ	0002h	; If set, lrHorizScroll is enabled
