; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

goToEditorOrDisasm:
#ifdef ENABLE_DISASM
	ld	a, (mflags + editorFlags)
	bit	retDisasm, a
	jp	nz, Disassembler
#endif
	jr	HexEditor

;====== Main Stuff =============================================================
hexEditorGoToFreeRAM:
	ld	hl, stringFindFreeRAM
	ld	(editLocation), hl

HexEditor:
	ld	a, (mflags + editorFlags)
	and	retDisasmMask^255
	ld	(mflags + editorFlags), a
EditFullRedraw:
	call	SetFullScrnWind
	call	HomeUp
	ld	hl, (editLocation)
	ld	d, textRows-1 ; -1 for index-from-zero, -1 for status bar
editLineLoop:
	ld	bc, (editOffset)
	add	hl, bc
	call	DispHL
	or	a
	sbc	hl, bc
	ld	a, 58 ; colon
	call	PutC
	call	PutSpace
	ld	e, 4
_:	ld	a, (hl)
	inc	hl
	call	DispByte
	dec	e
	jr	nz, -_
	call	PutSpace
	ld	e, 4
_:	ld	a, (hl)
	inc	hl
	call	DispByte
	dec	e
	jr	nz, -_
	call	PutSpace
	ld	b, 8
_:	dec	hl
	djnz	-_
	ld	e, 8
_:	ld	a, (hl)
	inc	hl
	call	PutC2
	dec	e
	jr	nz, -_
	dec	d
	jr	nz, editLineLoop
	; Status
	call	editShowMemMap
	call	editShowIndics
	
;------ Main Editor Loop -------------------------------------------------------
editLoop:
	call	editShowIndics
	call	editShowEffLoc
	call	setCurLoc
	call	CursorOn
	call	GetKey
	call	CursorOff
	ld	hl, editorKeyJumpTable
	call	ProcessKeyJumpTable
	jr	editLoop


;------ Status Bar: Memory Map -------------------------------------------------
editShowMemMap:
	ld	hl, textRows-1
	call	Locate
	in	a, (pMPgAHigh)
	call	DispByte
	in	a, (pMPgA)
	call	DispByte
	call	PutSpace
	in	a, (pMPgBHigh)
	call	DispByte
	in	a, (pMPgB)
	call	DispByte
	call	PutSpace
	ld	a, 'R'
	call	PutC
	in	a, (pMPgC)
	add	a, '0'
	call	PutC
	call	PutSpace
	ret

;------ Status Bar: Indicators -------------------------------------------------
editShowIndics:
; 123456789 123456789 123456789 12
; AAaa BBbb Rr BC ---- A DDEE:HHLL
	ld	hl, 13*256 + (textRows-1)
	call	Locate
	ld	a, (mflags + instrFlags)
	bit	noBcall, a
	jr	nz, +_
	ld	a, 'B'
	call	PutC
	ld	a, 'C'
	call	PutC
	jr	++_
_:	call	PutSpace
	call	PutSpace
_:	call	PutSpace
	ld	hl, (editOffset)
	ld	a, l
	or	h
	jr	z, +_
	call	DispHL
	jr	++_
_:	call	PutSpace
	call	PutSpace
	call	PutSpace
	call	PutSpace
_:	call	PutSpace
	ret
	
;------ Status Bar: Effective Location -----------------------------------------
editShowEffLoc:
; 0123456789 123456789 123456789 1
; AAaa BBbb Rr BC ---- A DDEE:HHLL
	ld	hl, 21*256 + (textRows-1)
	call	Locate
	call	compEffLoc
	ld	a, h
	rlca
	rlca
	and	3
	or	a
	jr	z, +_
	add	a, 'A'-1
_:	call	PutC
	call	PutSpace
	ld	a, (editLocation+1)
	cp	0C0h
	jr	nc, editShowEffLocNotPaged
	cp	40h
	jr	c, editShowEffLocNotPaged
	ld	c, pMPgAHigh
	cp	80h
	jr	nc, +_
	inc	c
_:	in	a, (c)
	call	DispByte
	res	3, c
	in	a, (c)
	call	DispByte
	ld	a, ':'
	call	PutC
	jr	+_
editShowEffLocNotPaged:
	call	PutSpace
	call	PutSpace
	call	PutSpace
	call	PutSpace
	call	PutSpace
_:	call	compEffLoc
	ld	de, (editOffset)
	add	hl, de
	call	DispHL
	ret

editShowMessage:
	push	hl
	ld	hl, 21*256 + (textRows-1)
	call	Locate
	pop	hl
	call	PutS
#ifdef ENABLE_DISASM
	ld	a, (mflags + editorFlags)
	bit	retDisasm, a
	jp	nz, disasmLoop
#endif
	jp	editLoop

;------ Cursor Location --------------------------------------------------------
setHexCurLoc:
; Moves the cursor to the location of the current byte under the cursor.
; Destroys:
;  - AF
	ld	a, (editXLoc)
	add	a, a
	cp	8
	jr	c, +_
	inc	a
_:	add	a, 6
	ld	h, a
	ld	a, (editYLoc)
	ld	l, a
	jp	Locate

setCharCurLoc:
; 123456789 123456789 123456789 12
; xxxx: 12345678 12345678 12345678
	ld	a, (editYLoc)
	ld	l, a
	ld	a, (editXLoc)
	add	a, 26
	ld	h, a
	jp	Locate

setCurLoc:
	ld	a, (editXLoc)
	cp	8
	jr	c, +_
	xor	a
	ld	(editXLoc), a
_:	ld	a, (editYLoc)
	cp	textRows-1
	jr	c, +_
	xor	a
	ld	(editYLoc), a
_:	ld	a, (mflags + editorFlags)
	bit	hexTexsMode, a
	jr	z, setHexCurLoc
	jr	setCharCurLoc


;------ compEffLoc -------------------------------------------------------------
; Computes the address under the cursor.
; Inputs:
;  - None
; Output:
;  - HL
; Destroys:
;  - AF
compEffLoc:
	ld	a, (editYLoc)
	add	a, a
	add	a, a
	add	a, a
	ld	b, a
	ld	a, (editXLoc)
	add	a, b
	ld	hl, (editLocation)
	add	a, l
	ld	l, a
	ret	nc
	inc	h
	ret


;------ Cursor Movement --------------------------------------------------------
shiftForward:
	ld	hl, (editLocation)
	inc	hl
	jr	setLoc

shiftBackward:
	ld	hl, (editLocation)
	dec	hl
	jr	setLoc

moveForward:
	ld	a, (editYLoc)
	cp	textRows-1-1
	jr	nz, +_
	ld	a, (editXLoc)
	cp	7
	jr	nz, ++_
	ld	hl, (editLocation)
	ld	de, 64
	add	hl, de
	jr	setLoc
_:	add	a, 8
	cp	textRows-1-1
	jr	nc, +_
	ld	(editYLoc), a
	jp	editLoop
_:	ld	hl, (textRows-1-1)*256 + 7
	ld	(editXLoc), hl
	jp	editLoop

moveLeft:
	ld	a, (editXLoc)
	or	a
	jr	z, +_
	dec	a
	ld	(editXLoc), a
	jp	editLoop
_:	ld	a, 7
	ld	(editXLoc), a
;	jr	moveUp
moveUp:
	ld	a, (editYLoc)
	or	a
	jr	z, +_
	dec	a
	ld	(editYLoc), a
	jp	editLoop
_:	ld	hl, (editLocation)
	ld	de, -8
	add	hl, de
setLoc:	ld	(editLocation), hl
	jp	EditFullRedraw

moveRight:
	ld	a, (editXLoc)
	cp	7
	jr	z, +_
	inc	a
	ld	(editXLoc), a
	jp	editLoop
_:	xor	a
	ld	(editXLoc), a
;	jr	moveDown
moveDown:
	ld	a, (editYLoc)
	cp	textRows-1-1
	jr	z, +_
	inc	a
	ld	(editYLoc), a
	jp	editLoop
_:	ld	hl, (editLocation)
	ld	de, 8
	add	hl, de
	jr	setLoc
	
moveBackward:
	ld	a, (editYLoc)
	or	a
	jr	nz, +_
	ld	a, (editXLoc)
	or	a
	jr	nz, ++_
	ld	hl, (editLocation)
	ld	de, -64
	add	hl, de
	jr	setLoc
_:	sub	8
	jr	c, +_
	ld	(editYLoc), a
	jp	editLoop
_:	ld	hl, 0
	ld	(editXLoc), hl
	jp	editLoop

jumpForward:
	ld	a, (editLocation+1)
	inc	a
	ld	(editLocation+1), a
	jp	goToEditorOrDisasm

jumpBackward:
	ld	a, (editLocation+1)
	dec	a
	ld	(editLocation+1), a
	jp	goToEditorOrDisasm

bigJumpForward:
	ld	a, (editLocation+1)
	add	a, 10h
	ld	(editLocation+1), a
	jp	goToEditorOrDisasm

bigJumpBackward:
	ld	a, (editLocation+1)
	sub	10h
	ld	(editLocation+1), a
	jp	goToEditorOrDisasm

	
;------ Go to Address ----------------------------------------------------------
gotoAddress:
; 123456789 123456789 123456789 12
; AAaa BBbb Rr BC ---- A DDEE:HHLL
	call	stashCurrentAddress
	ld	hl, 0
	call	Locate
	call	GetHexByte
	push	af
	call	GetHexByte
	ld	l, a
	pop	af
	ld	h, a
	ld	de, (editOffset)
	or	a
	sbc	hl, de
	ld	(editLocation), hl
	jp	goToEditorOrDisasm

hexEditPage:
	call	stashCurrentAddress
	ld	hl, textRows-1
	call	Locate
	call	GetHexByte
	out	(pMPgAHigh), a
	call	GetHexByte
	out	(pMPgA), a
	jp	goToEditorOrDisasm


; 123456789 123456789 123456789 12
; AAaa BBbb Rr BC ---- A No more.
; AAaa BBbb Rr BC ---- A Bookmarkd
; AAaa BBbb Rr BC ---- A Find:1234
; F1: Options
; F2: Find options
; F3: Refresh
; F4: Backward
; F5: Forward

;------ Editing ----------------------------------------------------------------
editByte:
	ld	hl, 23*256 + (textRows-1)
	call	Locate
	ld	hl, strWrite
	call	PutS
	call	setCurLoc
	ld	a, (mflags + editorFlags)
	bit	hexTexsMode, a
	jr	nz, +_
	call	GetHexByte
	jr	editByteCheck
_:	ld	a, 0Fh
	ld	(mflags + inputerFlags), a
	call	GetAscii
editByteCheck:
	push	af
	call	compEffLoc
	pop	af
	ld	(hl), a
	push	hl
	call	setHexCurLoc
	pop	hl
	ld	a, (hl)
	call	DispByte
	push	af
	call	setCharCurLoc
	pop	af
	call	PutC2
	jp	editLoop


;------ Bookmarks --------------------------------------------------------------

stashCurrentAddress:
; Stores the current page/address mapping into bookmark 0.
; Destorys:
;  - AF, BC, HL
	ld	hl, editBookmark0
storeCurrentAddress:
; Writes the current page/address mapping into the given bookmark struct
; Input:
;  - HL: Pointer to struct to write to
; Destroys:
;  - AF
;  - BC
;  - HL
	ld	bc, (editLocation)
	ld	(hl), c
	inc	hl
	ld	(hl), b
	inc	hl
	ld	c, pMPgA
	bit	7, b
	jr	z, +_
	inc	c
_:	in	a, (c)
	ld	(hl), a
	inc	hl
	set	3, c
	in	a, (c)
	ld	(hl), a
	inc	hl
	ld	bc, (editXLoc)
	ld	(hl), c
	inc	hl
	ld	(hl), b
	ret

exchangeCurrentAddressWithBookmark0:
; Swaps the current location with the location in bookmark 0.
; Uses the stack for scrap.
; Destorys:
;  - AF, BC, DE, HL
	ld	hl, editBookmark0+sizeOfBkm-1
	ld	d, (hl)
	dec	hl
	ld	e, (hl)
	dec	hl
	push	de
	ld	d, (hl)
	dec	hl
	ld	e, (hl)
	dec	hl
	push	de
	ld	d, (hl)
	dec	hl
	ld	e, (hl)
	push	de
	ld	hl, 0
	add	hl, sp
	call	loadBookmark
	pop	af
	pop	af
	pop	af
	jp	goToEditorOrDisasm

loadBookmark:
; Input:
;  - HL: Pointer to bookmark struct
	push	hl
	call	stashCurrentAddress
	pop	hl
	ld	e, (hl)
	inc	hl
	ld	d, (hl)
	inc	hl
	ld	(editLocation), de
	ld	a, d
	cp	40h
	jr	c, restoreAddrNoRemap
	cp	0C0h
	jr	nc, restoreAddrNoRemap
	ld	c, pMPgA
	bit	7, d
	jr	z, +_
	inc	c
_:	ld	a, (hl)
	inc	hl
	out	(c), a
	set	3, c
	ld	a, (hl)
	inc	hl
	out	(c), a
	jr	+_
restoreAddrNoRemap:
	inc	hl
	inc	hl
_:	ld	a, (hl)
	inc	hl
	cp	textCols
	jr	nc, +_
	ld	(editXLoc), a
_:	ld	a, (hl)
	inc	hl
	cp	textRows
	ret	nc
	ld	(editYLoc), a
	ret

goBookmark9:
	inc	a
goBookmark8:
	inc	a
goBookmark7:
	inc	a
goBookmark6:
	inc	a
goBookmark5:
	inc	a
goBookmark4:
	inc	a
goBookmark3:
	inc	a
goBookmark2:
	inc	a
goBookmark1:
	ld	hl, editBookmarks
#if sizeOfBkm != 6
	.error	"Code in goBookmarkN must be adjusted to reflect correct size of bookmark struct."
#endif
	add	a, a
	ld	b, a
	add	a, b
	add	a, b
	add	a, l
	ld	l, a
	jr	nc, +_
	inc	h
_:	in	a, (pIntId)
	and	8
	jr	nz, +_
	; Save
	call	storeCurrentAddress
	; Show message
	ld	hl, strBookmarkSaved
	jp	editShowMessage
_:	call	loadBookmark
	jp	goToEditorOrDisasm


;------ Find -------------------------------------------------------------------

;------ Options ----------------------------------------------------------------
hexEditMenu:
	ld	hl, mHexEditOptions
	jp	Menu

editVal8:
editChangeOffset:
	