; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

;------ SetUpInterrupts --------------------------------------------------------
SetUpInterrupts:
; Sets up the IM 2 interrupt system, but does not enable interrupts.
; Inputs:
;  - None
; Output:
;  - Interrupts set up
; Destorys:
;  - Assume all
	di
	xor	a
	out	(pIntMask), a
	out	(pCrstlTmr1Freq), a
	out	(pCrstlTmr2Freq), a
	out	(pCrstlTmr3Freq), a
	out	(pLnkAstSeEnable), a
	; And kill USB
	out	(57h), a
	out	(5Bh), a
	out	(4Ch), a
	ld	a, 2
	out	(54h), a
	; Custom Interrupts
	ld	hl, IvtLocation*256
	ld	de, IvtLocation*256+1
	ld	bc, 256
	ld	a, IvtLocation
	ld	i, a
	ld	a, IsrLocation
	ld	(hl), a
	ldir
	ld	hl, InterruptServiceRoutine
	ld	de, IsrLocation*256+IsrLocation
	ld	c, (hl)
	inc	hl
	ld	b, (hl)
	inc	hl
	ldir
	; Enable only wanted interrupts
	;ld	a, tmrFreq0 | memMapMode0 | battVoltage0
	xor	a	; Above expression evaulates to 0
	out	(pMMapTmrFreq), a
	ld	a, intTmr2 | intOnKey | intDisableLowPowerMode
	out	(pIntMask), a
	im	2
	ret
InterruptServiceRoutine:
	.dw	3
	jp	realIsr
InterruptServiceRoutineEnd:


;------ Interrupt Service Routine ----------------------------------------------
realIsr:
	push	af
	; Using JP allows people to patch this table so they can patch in
	; their own interrupt sources.
	in	a, (pIntId)
	rr	a
	jp	c, intHOnKey
	rr	a
	call	c, RebootMicrOS;intApd
	rr	a
	jp	c, intHPapd
	rr	a
;	rr	a
;	jp	c, intHLink
;	rr	a
;	jp	c, intHCrystal
;	rr	a
;	jp	c, intHHCrystal
	; Wait, there should be no other active interrupt sources.
	call	RebootMicrOS
;intHLink:
;intHCrystal:
;intHApd: ; WTF, didn't we disable this interrupt?
;	call	RebootMicrOS
intHOnKey: ; ON key is for panicing!
	ld	a, 0FFh
	out	(pKey), a
	pop	af
	push	af
	pop	af
	push	af
	ld	a, 0BFh
	out	(pKey), a
	pop	af
	push	af
	pop	af
	push	af
	in	a, (pKey)
	cp	0BFh
	jp	z, QuitToEOS
	cp	7Fh
	jp	z, RebootMicrOS
intHPapd:
	push	bc
	; ACK the interrupt
	;ld	a, intTmr1 ^ 255
	xor	a
	out	(pIntAck), a
	; Check the cursor timer.  This timer is always running.
	ld	bc, (cursorTimer)
	dec	bc
	ld	(cursorTimer), bc
	ld	a, b
	or	c
	jr	nz, noCursorBlink
	ld	bc, (cursorTimerPeriod)
	ld	(cursorTimer), bc
	ld	a, (mflags + curFlags)
	or	curFlashMask
	ld	(mflags + curFlags), a
	; Check if we're sleepy
	ld	a, (mflags + appFlags)
	bit	apdEnabled, a
	jr	z, noCursorBlink
	ld	bc, (suspendTimer)
	dec	bc
	ld	(suspendTimer), bc
	ld	a, b
	or	c
	call	z, MicrosApd
noCursorBlink:
	; Check if it's time to scan the keyboard
	ld	a, (kbdScanTimer)
	dec	a
	ld	(kbdScanTimer), a
	jr	nz, noScan
;	jr	z, scanKbd
;	jr	noScan
scanKbd:
	ld	a, kbdScanDivisor
	ld	(kbdScanTimer), a
	call	ScanKeyboard
noScan:	pop	bc
	pop	af
	ei
	ret