; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

; It might be nice to have a GetString routine at some point.  Whatever.


;------ GetAscii ---------------------------------------------------------------
; Gets a single key press, with modifiers from shiftFlags processed.  2nd and
; Alpha flags are reset, unless alpha-lock is enabled, in which case only 2nd is
; reset.
; Inputs:
;  - inputerFlags
; Outputs:
;  - ASCII char in B, might be a control char
;  - D: Shift flags
;  - E: inputerFlags
; Destroys:
;  - Pretty much everything, most likely.
GetAscii:
	ld	a, (mflags + inputerFlags)
	and	1Fh
	ret	z
	call	GetKeyShifts
	bit	allowAlpha, e
	jr	z, getAsciiNum
	bit	shiftAlpha, d
	jr	z, getAsciiNum
	ld	hl, textAlphaTable
	call	getAsciiScanTable
	jr	z, +_
	bit	allowLwrAlpha, e
	jr	z, ClearShiftFlags
	bit	shiftLwrAlpha, d
	jr	z, ClearShiftFlags
	add	a, 20h
	jr	ClearShiftFlags
_:	ld	hl, textAlphaSymbolTable
	bit	allowSymbols, e
	jr	z, GetAscii
	call	getAsciiScanTable
	jr	nz, ClearShiftFlags
	jr	getAsciiControlChar
getAsciiNum:
	ld	hl, textNumberSymbolTable
	bit	allowNumbers, e
	jr	z, getAsciiControlChar
	ld	hl, textNumberTable
	call	getAsciiScanTable
	jr	nz, ClearShiftFlags
	ld	hl, textNumberSymbolTable
	bit	allowSymbols, e
	call	nz, getAsciiScanTable
	jr	nz, ClearShiftFlags
getAsciiControlChar:
	bit	allowControl, e
	jr	z, GetAscii
	bit	shift2nd, d
	jr	z, +_
	ld	hl, text2ndControlCharTable
getAsciiCtrl:
	call	getAsciiScanTable
	jr	nz, ClearShiftFlags
	jr	GetAscii
_:	bit	shiftAlpha, d
	jr	z, +_
	ld	hl, textAlphaControlCharTable
	jr	getAsciiCtrl
_:	ld	hl, textControlCharTable
	jr	getAsciiCtrl

	
getAsciiScanTable:
	ld	b, (hl)
	inc	hl
	ld	c, (hl)
	inc	hl
	ld	h, (hl)
	ld	l, c
_:	cp	(hl)
	inc	hl
	jr	z, +_
	inc	hl
	djnz	-_
	cp	b
	ret
_:	ld	b, (hl)
	cp	a
	ret


;------ ClearShiftFlags --------------------------------------------------------
ClearShiftFlags:
; Clears 2nd and Alpha.  However, if Alpha Lock is on, alpha is not cleared.  If
; a special cursor is selected, it is not cleared.  See ClearAllShiftFlags for a
; routine to clear all shift flags.
; Inputs:
;  - None
; Output:
;  - Flags cleared
; Destroys:
;  - AF
	ld	a, (mflags + shiftFlags)
	bit	shiftOther, a
	ret	nz
	and	shift2ndMask ^ 255
	ld	(mflags + shiftFlags), a
	bit	shiftALock, a
	ret	nz
	and	(shiftLwrAlphaMask | shiftAlphaMask) ^ 255
	ld	(mflags + shiftFlags), a
	ret


;------ ClearAllShiftFlags -----------------------------------------------------
ClearAllShiftFlags:
; Inputs:
;  - None
; Output:
;  - Flags cleared
; Destroys:
;  - AF
	ld	a, (mflags + shiftFlags)
	and	(shiftALockMask | shiftLwrAlphaMask | shiftAlphaMask | shift2ndMask | shiftInsertMask | shiftOtherMask) ^ 255
	ld	(mflags + shiftFlags), a
	ret


;------ GetKeyShifts -----------------------------------------------------------
GetKeyShifts:
; Gets a single key press.  If the cursor is enabled, it will blink, and change
; according to shiftFlags.
; Input:
;  - shiftFlags you want preset
; Output:
;  - Key code in A
;  - Flags
getAsciiLoop:
	di
	ld	bc, (cursorTimerPeriod)
	ld	(cursorTimer), bc
	ei
	call	GetKey
	ld	l, a
	ld	a, (mflags + inputerFlags)	; E: inputerFlags
	ld	e, a
	ld	a, (mflags + shiftFlags)	; D: shiftFlags
	ld	d, a
	ld	a, l
	; Check cursor options
	cp	skAlpha
	jr	z, getKeyNoAlpha
	bit	allowAlpha, e
	jr	z, getAsciiLoop
	ld	a, d
	bit	shift2nd, a
	jr	z, +_
	and	shift2ndMask ^ 255
	or	shiftAlphaMask
	jr	getAsciiSetFlags
_:	bit	shiftAlpha, a
	jr	nz, +_
	or	shiftAlphaMask
	and	shiftLwrAlphaMask ^ 255
	jr	getAsciiSetFlags
_:	bit	allowLwrAlpha, d
	jr	nz, +_
_:	and	(shiftAlphaMask | shiftLwrAlphaMask) ^ 255
	jr	getAsciiSetFlags
_:	bit	shiftLwrAlpha, a
	jr	nz, --_
	or	shiftLwrAlphaMask	
getAsciiSetFlags:
	ld	(mflags + shiftFlags), a
	jr	getAsciiLoop
getKeyNoAlpha:
	cp	sk2nd
	ret	nz
	ld	a, d
	bit	shift2nd, a
	jr	nz, +_
	or	shift2ndMask
	jr	getAsciiSetFlags
_:	and	shift2ndMask ^ 255
	jr	getAsciiSetFlags


;------ Hex input --------------------------------------------------------------
GetHexByte:
; Do not attempt to get a byte across a line break.
; Gets byte at current location.  Returns byte in A
; Registers:
; b: returns nibble from getNibble
; d: byte
	push	bc
	push	de
	push	hl
	call	CursorOn
	call	getNibble
	sla	b
	sla	b
	sla	b
	sla	b
	push	bc
	call	getNibble
	ld	a, b
	pop	bc
	add	a, b
	push	af
	call	CursorOff
	pop	af
	pop	hl
	pop	de
	pop	bc
	ret
getNibble:
	call	GetKey
	ld	hl, hexKeyTable
	ld	b, hexKeyTableSize
_:	cp	(hl)
	jr	z, foundNibble
	inc	hl
	djnz	-_
	jr	getNibble
foundNibble:
	dec	b
	ld	a, '0'
	add	a, b
	cp	3Ah
	jr	c, +_
	add	a, 7
_:	call	PutC
	ret


;------ GetKey -----------------------------------------------------------------
GetKey:
; Waits for a keypress, and returns it.
; This may APD.
; Interrupts must be enabled.
; Inputs:
;  - None
; Output:
;  - Scan code in A
; Destroys:
;  - Nothing
	call	GetCSC
	or	a
	jr	nz, EraseCursor
	ei
	halt
	ld	a, (mflags + curFlags)
	bit	curAble, a
	jr	z, GetKey
	bit	curFlash, a
	jr	z, GetKey
	call	cursorToggle
	jr	GetKey
EraseCursor:
	push	af
	ld	a, (mflags + curFlags)
	bit	curAble, a
	jr	z, +_
	bit	curOn, a
	call	nz, CursorToggle
_:	pop	af
	ret
CursorToggle:
	push	bc
	push	de
	push	hl
	ld	a, (mflags + curFlags)
	and	curFlashMask ^ 255
	bit	curOn, a
	jr	nz, cursorErase
cursorShow:
	or	curOnMask
	ld	(mflags + curFlags), a
	call	ReadChar
	ld	(cursorChar), a
	call	FixCursor
	ld	a, (mflags + shiftFlags)
	ld	c, 0
	ld	b, a
	ld	a, chCur
	bit	shiftOther, b
	jr	z, cursorNormalSet
	; Handle other cases
	ld	a, chCurFull
	bit	shiftFull, b
	jr	nz, cursorShowCursor
;	ld	a, 
;	bit	shiftBox, b
	jr	cursorShowCursor
cursorNormalSet:
	bit	shiftInsert, b
	jr	z, +_
	add	a, 4
_:	bit	shift2nd, b
	jr	z, +_
	inc	a
	jr	cursorShowCursor
_:	bit	shiftAlpha, b
	jr	z, cursorShowCursor
	add	a, 2
	bit	shiftLwrAlpha, b
	jr	z, cursorShowCursor
	inc	a	
cursorShowCursor:
	ld	hl, (currentRow)
	push	hl
	call	PutC
	;call	BackspaceCursor
	pop	hl
	call	Locate
	jr	cursorToggleRet
cursorErase:
	and	curOnMask ^ 255
	ld	(mflags + curFlags), a
	; Erase the cursor
#if textInverse != 0
	.error	"Change cursorErase to know the correct value of textInverse"
#endif
	ld	a, (cursorChar)
	and	80h
	rlc	a
	ld	b, a
	ld	a, (mflags + textFlags)
	ld	c, a
	and	textInverseMask^255
	or	b
	ld	(mflags + textFlags), a
	ld	a, (cursorChar)
	and	7Fh
	ld	hl, (currentRow)
	push	hl
	call	PutC
	pop	hl
	call	Locate
	;call	BackspaceCursor
	ld	a, c
	ld	(mflags + textFlags), a
cursorToggleRet:
	pop	hl
	pop	de
	pop	bc
	ret


;------ CursorOn ---------------------------------------------------------------
CursorOn:
; Enables the blinking cursor that GetKey can show
; Inputs:
;  - None
; Output:
;  - Documented effect(s)
; Destroys:
;  - Nothing
	push	af
	ld	a, (mflags + curFlags)
	or	curAbleMask | curFlashMask
;	and	curOnMask ^ 255
	ld	(mflags + curFlags), a
;	call	CursorToggle;cursorShow
	push	bc
	ld	bc, (cursorTimerPeriod)
	ld	(cursorTimer), bc
	pop	bc
	pop	af
	ret


;------ CursorOff --------------------------------------------------------------
CursorOff:
; Disables the blinking cursor
; Inputs:
;  - None
; Output:
;  - Documented effect(s)
; Destroys:
;  - Nothing
	push	af
	ld	a, (mflags + curFlags)
	and	(curAbleMask) ^ 255
	ld	(mflags + curFlags), a
;	call	EraseCursor
	pop	af
	ret


;------ GetCSC -----------------------------------------------------------------
GetCSC:
; Returns the most recent keypress and removes it from the key queue.
; Inputs:
;  - None
; Output:
;  - Scan code in A
; Destorys:
;  - Nothing
	ld	a, (keyBuffer)
	push	af
	xor	a
	ld	(keyBuffer), a
	pop	af
	ret


;------ PeekCSC ----------------------------------------------------------------
PeekCSC:
; Returns the most recent keypress but leaves it in the key queue.
; Inputs:
;  - None
; Output:
;  - Scan code in A
; Destroys:
;  - Nothing
	ld	a, (keyBuffer)
	ret


;------ Scan Keyboard ----------------------------------------------------------
ScanKeyboard:
; Implements the core logic used for keyboard scanning, including timing for
; debouncing.
; You can call this manually if interrupts are disabled, but you will need to do
; so at about 100 Hz or the debouncing timing will be strange.
; Inputs:
;  - None
; Output:
;  - Documented effect(s)
; Destroys:
;  - AF
;  - BC
	call	RawGetCSC
; Key repeat logic:
; Key must be held for at least minKeyHoldTime before press registers.
; Thereafter, the same key code will not be accepted after release for keyBlockTime.
; As long as the key is being held, reissue the keypress to keyBuffer every keyRepeatWaitTime.
	; Any code?
	or	a
	jr	nz, dontReleaseKey
	ld	a, (lastKey)
	or	a
	ret	z	;jr	z, noKeys
	; Otherwise, begin blocking timer
	ld	a, (lastKeyBlockTimer)
	dec	a
	ld	(lastKeyBlockTimer), a
	ret	nz	;jr	nz, nokeys
	; The timer has expired, so allow the key again
	ld	(lastKey), a	; already 0
	ret
dontReleaseKey:
	ld	c, a	; Save code for a moment
	; Reset the blocking timer
	ld	a, keyBlockTime
	ld	(lastKeyBlockTimer), a
	; Is the registered key the same as the last key?
	; This also covers the 0 case.
	ld	a, (lastKey)
	cp	c
	jr	nz, newKey	; If not, accept the new key
	; It's the same old key, so decrement the hold timer
	ld	a, (lastKeyHoldTimer)
	or	a
	jr	nz, decHoldTimer
	; Wait, the key has already been accepted.
	; Check if we're already in repeat mode, or need to wait
	ld	a, (lastKeyFirstRepeatTimer)
	or	a
	jr	z, +_	; Repeat mode
	dec	a
	ld	(lastKeyFirstRepeatTimer), a
	ret	;jr	noKeys
_:	; OK, so we're in repeat mode.  Now do the reapeat timer.
	ld	a, (lastKeyRepeatTimer)
	or	a
	jr	z, acceptKeypress	; The key was previously accepted, AND it's now ready to be reissued.
	dec	a
	ld	(lastKeyRepeatTimer), a
	ret
decHoldTimer:
	dec	a
	ld	(lastKeyHoldTimer), a
	jr	z, acceptKeypress
	; This key isn't ripe yet, so ignore it for now.
	ret	;jr	noKeys
newKey:	; Save the key code
	ld	a, c
	ld	(lastKey), a
	; Reset timers
	ld	bc, keyBlockTime*256 + minKeyHoldTime
	ld	(lastKeyHoldTimer), bc
	ld	bc, keyRepeatWaitTime*256 + keyFirstRepeatWaitTime
	ld	(lastKeyFirstRepeatTimer), bc
	; Now RET, and leave decrementing the debouncing timers for the next
	; interrupt cycle
	ret	;jr	noKeys
acceptKeypress:
	; Reset timers
	ld	a, keyBlockTime
	ld	(lastKeyBlockTimer), a
	ld	a, keyRepeatWaitTime
	ld	(lastKeyRepeatTimer), a
	ld	bc, suspendDelay
	ld	(suspendTimer), bc
	; Now write the key to the buffer
	ld	a, (lastKey)
	ld	(keyBuffer), a
;noKeys:	ret
	ret


;------ GetCSC -----------------------------------------------------------------
RawGetCSC:
; Scans the keyboard matrix for any pressed key, returning the first it finds,
; or 0 if none.
; Inputs:
;  - None
; Output:
;  - Code in A, or 0 if none
; Destroys:
;  - BC
	ld	a, 0FFh
	out	(pKey), a
	pop	af
	push	af
	ld	bc, 07BFh
_:	; Matrix scan loop
	ld	a, c
	out	(pKey), a
	rrca
	ld	c, a
	pop	af	; Probably should waste at least 20 cycles here.
	push	af	
;	push	ix
;	pop	ix
	in	a, (pKey)
	cp	0ffh
	jr	nz, +_	; Any key pressed?
	djnz	-_
	; No keys pressed in any key group, so return 0.
	xor	a
	ret
_:	; Yay! Found a key, now form a scan code
	dec	b
	sla	b
	sla	b
	sla	b
	; Get which bit in A is reset
_:	rrca
	inc	b
	jr	c, -_
	ld	a, b
	ret
