; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

;------ IncPtr -----------------------------------------------------------------
IncPtr:
; Increments DEHL, wrapping as needed.  If HL points outside of the swappable
; memory bank, this does not attempt any wrapping.
; Input:
;  - DEHL
; Output:
;  - DEHL+1
; Destroys:
;  - AF
	inc	l
	ret	nz
	inc	h
	ld	a, h
	cp	swapTop >> 8
	ret	nz
	sub	40h
	ld	h, a
	inc	e
	ld	a, e
	cp	80h
	ret	c
	ld	e, 0
	inc	d
	ret


;------ DecPtr -----------------------------------------------------------------
DecPtr:
; Decrements DEHL, wrapping as needed.  If HL points outside of the swappable
; memory bank, this does not attempt any wrapping.
; Input:
;  - DEHL
; Output:
;  - DEHL+1
; Destroys:
;  - AF
	ld	a, l
	dec	l
	or	a
	ret	nz	; If L != 0 before, L does not underflow and H is fine.
	ld	a, h
	dec	h
	cp	swapLoc >> 8
	ret	nz
	ld	a, e
	dec	e
	or	a
	jr	z, +_
	cp	80h
	ret	nz
	ld	e, 87h
	ret
_:	ld	e, 7Fh
	ld	a, d
	dec	a
	and	3
	ld	d, a
	ret


;------ AddPtr -----------------------------------------------------------------
AddPtr:
; Adds BC to DEHL, wrapping as needed.  THIS WILL SCREW UP if HL points outside
; of the swap bank.
; Inputs:
;  - DEHL: Pointer
;  - BC: Amount to add to pointer
; Output:
;  - DEHL+=BC
; Destroys:
;  - AF
#if swapLoc != 4000h
	.error	"Oops, fix the AddPtr function.  It assumes swapping happens at 4000h."
#endif
	ld	a, l
	add	a, c
	ld	l, a
	ret	nc
	ld	a, h
	add	a, b
	ld	h, a
	call	c, addPtrIncDe
	ld	a, h
	cp	40h
	call	c, addPtrIncDe
	ld	a, h
	cp	0C0h
	call	nc, addPtrIncDe
	ld	a, h
	cp	80h
	ret	c
addPtrIncDe:
	ld	a, h
	sub	40h
	ld	h, a
	inc	e
	ld	a, e
	cp	88h
	jr	nc, +_
	cp	80h
	ret	nz
	ld	e, 0
	inc	d
	ret
_:	ld	e, 80h
	ret


;------ SubPtr -----------------------------------------------------------------
SubPtr:
; Subtracts BC from DEHL, wrapping as needed.  THIS WILL SCREW UP if HL points
; outside of the swap bank.
; Inputs:
;  - DEHL: Pointer
;  - BC: Amount to subtract from pointer
; Output:
;  - DEHL-=BC
; Destroys:
;  - AF
#if swapLoc != 4000h
	.error	"Oops, fix the SubPtr function.  It assumes swapping happens at 4000h."
#endif
	ld	a, l
	sub	c
	ld	l, a
	ret	nc
	ld	a, h
	sub	b
	ld	h, a
	call	c, subPtrDecDe
	ld	a, h
	cp	80h
	call	c, subPtrDecDe
	ld	a, h
	cp	0C0h
	call	c, subPtrDecDe
	ld	a, h
	cp	040h
	ret	nc
subPtrDecDe:
	ld	a, h
	add	a, 40h
	ld	h, a
	dec	e
	ld	a, e
	cp	7Fh
	jr	z, +_
	cp	0FFh
	ret	nz
	ld	e, 7Fh
	dec	d
	ret
_:	ld	e, 87h
	ret


;------ GetByte ----------------------------------------------------------------
GetByte:
; Gets a byte from the given location.
; Input:
;  - DEHL: Address
; Output:
;  - A: Byte returned
; Destroys:
;  - Nothing
	push	bc
	in	a, (swapPort)
	push	af
	in	a, (swapPortHigh)
	push	af
	ld	a, e
	out	(swapPort), a
	ld	a, d
	out	(swapPortHigh), a
	ld	b, (hl)
	pop	af
	out	(swapPortHigh), a
	pop	af
	out	(swapPort), a
	ld	a, b
	pop	bc
	ret


;------ PutByte ----------------------------------------------------------------
PutByte:
; Writes a byte to the given address, which should be RAM.  If it's not RAM, the
; write will probably be ignored.
; Input:
;  - DEHL: Address
;  - A: Byte to write
; Output:
;  - Byte written
; Destroys:
;  - AF
	push	bc
	ld	b, a
	in	a, (swapPort)
	push	af
	in	a, (swapPortHigh)
	push	af
	ld	a, e
	out	(swapPort), a
	ld	a, d
	out	(swapPortHigh), a
	ld	(hl), b
	pop	af
	out	(swapPortHigh), a
	pop	af
	out	(swapPort), a
	pop	bc
	ret


;====== Flash Commands =========================================================
;------ Block Copy/Write -------------------------------------------------------
BlockCopy:
; Copies up to 64 K of data from one location to another.  If B is set to
; BlockCopyFlashWriteMode, then the writes will be treated as flash writes.
; Inputs:
;  - B': Mode
;  - DEHL: Source address
;  - DEHL': Destination address
;  - IX: Bytes to copy
; Output:
;  - Data copied
; Destroys:
;  - AF, AF'
;  - BC, BC'
;  - DE, DE'
;  - HL, HL'
;  - IX
; I couldn't think of a use for IY here.
BlockCopyFlashWriteMode	.equ	1
BlockCopyNoFlashWrite	.equ	0
blockCopyWriteFlash	.equ	0
; A: Read val
; B': Mode
; A': Temp for flash write seq
; DEHL: Read/Write address
	res	1, b
	exx
	set	1, b
	exx
blockCopyLoop:
	; Read source byte
	ld	a, e
	out	(swapPort), a
	ld	a, d
	out	(swapPort|8), a
	ld	a, (hl)
	; Map destination into memory
	exx
	ld	c, a
	ld	a, e
	out	(swapPort), a
	ld	a, d
	out	(swapPort|8), a
	bit	blockCopyWriteFlash, b
	jr	z, noWrite
	; Write-to-flash
	ld	a, 0AAh	; First bus cycle---unlock
	ld	(0AAAh), a
	ld	a, 55h	; Second bus cycle---unlock
	ld	(0555h), a
	ld	a, 0A0h	; Third bus cycle---write command
	ld	(0AAAh), a
	ld	(hl), c	; Fourth bus cycle---program data
_:	ld	a, (hl)
	xor	c
	bit	7, a
	jr	z, blockCopyIncPtrs
	xor	c	; Don't read twice -> Prevent false failure notification
	bit	5, a	; if bit 5 just happens to be set in the good data.
	jr	z, -_
	; Error!
	ld	a, 0F0h
	ld	(0000h), a
noWrite:
	ld	(hl), c
blockCopyIncPtrs:
	; Inc pointers
	exx
blockCopyIncPtrs2:
	inc	l
	jr	nz, +_
	inc	h
	ld	a, h
	cp	swapTop >> 8
	jr	nz, +_
	sub	40h
	ld	h, a
	inc	e
	ld	a, e
	cp	80h
	jr	c, +_
	ld	e, 0
	inc	d
_:	bit	1, b
	exx
	jr	nz, blockCopyIncPtrs2
checkIfDone:
	; Are we done?
	dec	ix
	ld	a, ixh
	or	ixl
	jr	nz, blockCopyLoop
	ret


;------ Flash write ------------------------------------------------------------
WriteFlashByte:
; Writes a byte to flash.  Flash must be unlocked.  Be aware that pressing CLEAR
; will abort the write.
; Inputs:
;  - B: Byte to write
;  - D: Page High Bits
;  - E: Page
;  - HL: Address to write to.  This will not be wrapped.
; Outputs:
;  - NZ on failure
; Destroys:
;  - AF, BC
	; Map address into Z80 memory
	in	a, (pMPgA)
	push	af
	in	a, (pMPgAHigh)
	push	af
	ld	a, e
	out	(pMPgA), a
	ld	a, d
	out	(pMPgAHigh), a
	; Flash command sequence
	ld	a, 0AAh	; First bus cycle---unlock
	ld	(0AAAh), a
	ld	a, 55h	; Second bus cycle---unlock
	ld	(0555h), a
	ld	a, 0A0h	; Third bus cycle---write command
	ld	(0AAAh), a
	ld	(hl), b	; Fourth bus cycle---program data
	; Wait for data to be good
	ld	a, 0FDh		; This checks for the CLEAR key.
	out	(pKey), a		; If pressed, it aborts.
	inc	ix
	dec	ix
_:	in	a, (pKey)
	cp	0BFh
	jr	z, +_
	ld	a, (hl)
	xor	b
	bit	7, a
	jr	z, programDone
	xor	b
	bit	5, a	; Don't read twice -> Prevent false failure notification if bit 5 just happens to be set in the good data.
	jr	z, -_
_:	ld	a, 0F0h
	ld	(0000h), a
	inc	a
programDone:
	; Fix mappings
	pop	bc
	ld	a, b
	out	(pMPgA), a
	pop	bc
	ld	a, b
	out	(pMPgAHigh), a
	ret


;------ Erase sector -----------------------------------------------------------
EraseSector:
; Erases a sector of flash.
; Inputs:
;  - D: Page High Bits
;  - E: Page
;  - HL: Address (usually 4000h)
; Outputs:
;  - Z on fail (this is opposite of writeFlashByte)
; Destroys:
;  - AF, BC, HL
	; Map address into Z80 memory
	in	a, (pMPgA)
	push	af
	in	a, (pMPgAHigh)
	push	af
	ld	a, e
	out	(pMPgA), a
	ld	a, d
	out	(pMPgAHigh), a
	; Flash command sequence
	ld	a, 0AAh	; First bus cycle---unlock
	ld	(0AAAh), a
	ld	a, 55h	; Second bus cycle---unlock
	ld	(0555h), a
	ld	a, 080h	; Third bus cycle---write command
	ld	(0AAAh), a
	ld	a, 0AAh	; Fourth bus cycle---unlock (again)
	ld	(0AAAh), a
	ld	a, 55h	; Fifth bus cycle---unlock (again)
	ld	(0555h), a
	ld	(hl), 30h
	; Wait for data to be good
	ld	a, 0FDh
	out	(pKey), a
	inc	ix
	dec	ix        
_:	in	a, (pKey)
	cp	0BFh
	jr	z, +_
	cp	0EFh	; Allow the user to continue using MicrOS during the erase
	ret	z
	ld	a, (hl)
	bit	7, a
	jr	nz, eraseDone
	bit	5, a
	jr	z, -_
_:	ld	a, 0F0h
	ld	(4000h), a
	xor	a
eraseDone:	
	; Fix mappings
	pop	bc
	ld	a, b
	out	(pMPgA), a
	pop	bc
	ld	a, b
	out	(pMPgAHigh), a
	ret


;------ Autoselect Command Test ------------------------------------------------
IssueAutoselect:
; Inputs:
;  - HL: Address
;  - D: High Page Bits
;  - E: Page
; Output:
;  - B: Value read
	in	a, (pMPgA)
	push	af
	in	a, (pMPgAHigh)
	push	af
	ld	a, e
	out	(pMPgA), a
	ld	a, d
	out	(pMPgAHigh), a
	; First bus cycle---unlock
	ld	a, 0AAh
	ld	(0AAAh), a
	; Second bus cycle---unlock
	ld	a, 55h
	ld	(0555h), a
	; Third bus cycle---write command
	ld	a, 090h
	ld	(0AAAh), a
	; Read autoselect code
	ld	b, (hl) ;4001h 4047
	ld	a, 0F0h
	ld	(0000h), a
;	ld	l, a
	pop	af
	out	(pMPgAHigh), a
	pop	af
	out	(pMPgA), a
	ret
