;PutSClrWind appears to be destroying HL and DE.

; To do:
;  - Fix ScrollUp, which currently does nothing
;  - Switch to double-width font
;  - Need LCD initalization/uninitalization

;  - Editor:
;    - Find
;    - Options

; - Fix Milo.  Needs to be streamlined.  Doesn't need to do wacky self-deletion!


;===============================================================================
; MicrOS, a totally 80s, rudimentary OS.

; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details. 

; In this file:
;  - Compile time options
;  - Initalization stuff
;  - Quit/Exit/Suspend

; To do:
;  - Jump table

;====== Equates ================================================================
.nolist
#include "convhex.inc"
.list

;#define ENABLE_FULL_ERASE TRUE ; This is pretty dangerous.

; Version
MicrOSVersion	.equ	20h + 0
#define MicrOSVersionString	chBeta, "2.0"

; Space-saving options
;#define HEX_ASCII_CHART TRUE
#define ENABLE_DISASM TRUE
;#define ENABLE_USB_SUPPORT TRUE
; I think I wanted to move some buffers like textBuffer to the same 768-byte
; zone that the other main variables are stored in; but I never actually did it.
;#define NO_EXT_RAM TRUE
#define ENABLE_EOS_QUIT

.echo	" "
.echo	"MICROS ASSEMBLY OPTIONS:"
#ifdef ENABLE_FULL_ERASE
	.echo	"Full chip erase enabled."
#endif
#ifdef HEX_ASCII_CHART
	.echo	"Hex editor ASCII charted included."
#endif
#ifdef ENABLE_USB_SUPPORT
	.echo	"USB support enabled"
ENABLE_USB_SUPPORT_VAL .equ	8
#else
ENABLE_USB_SUPPORT_VAL .equ	0
#endif
#ifdef ENABLE_DISASM
	.echo	"Disassembler enabled."
#endif

#include "equates.asm"

.echo	" "


;====== Header =================================================================
	.org	0C080h-2
	.db	0D9h, 3Fh ; Stop, Newline
starting_address:

#include "jumptable.asm"

StartMicrOS:
	; Boot with HL = return address
	ld	(quitaddr), hl
	ld	(spaddr), sp
	in	a, (pMPgA)
	ld	(pageAOrig), a
	in	a, (pMPgAHigh)
	ld	(pageAOrig+1), a
	in	a, (pMPgB)
	ld	(pageBOrig), a
	in	a, (pMPgBHigh)
	ld	(pageBOrig+1), a
;	jp	RebootMicrOS

;====== Init ===================================================================

RebootMicrOS:
; Initalize Micros
initMOS:
; TODO: Initalize screen
	di
	ld	sp, topOfStack
	xor	a
	out	(pMemDelay), a
	inc	a	;ld	a, cpu15MHz
	out	(pCpuSpeed), a
	ld	a, crstlClockKeepAlive | crstlTmrKeepAlive
	out	(pCrstlKeepAlive), a
	ld	a, delayCycleMin
	out	(pLcdDelayCpu6MHz), a
	out	(pLcdDelayCpu15MHz), a
	out	(pLcdDelayCpu20MHz), a
	out	(pLcdDelayCpu25MHz), a
	; Memory map
	xor	a
	out	(pFlashExecLowerLimit), a
	out	(pFlashExecUpperLimit), a
	out	(pRamExecLowerLimit), a
	out	(pBlockMRemapC), a
	out	(pBlockMRemapB), a
	dec	a	; ld a, 0FFh
	out	(pRamExecUpperLimit), a
	out	(pFlashExecHighBit), a
	ld	a, flashType4MB; | ramType256K;ramType128K
	out	(pFlashRamType), a
	; Clear variables
	ld	sp, topOfStack
	ld	hl, start_of_static_vars
	ld	b, size_of_static_vars
	xor	a
	call	ClearMem
	call	SetUpInterrupts
	ld	bc, cursorTimerDefaultValue
	ld	(cursorTimerPeriod), bc
	ld	(cursorTimer), bc
	ld	a, kbdScanDivisor
	ld	(kbdScanTimer), a
	ld	bc, keyBlockTime*256 + minKeyHoldTime
	ld	(lastKeyHoldTimer), bc
	ld	bc, keyRepeatWaitTime*256 + keyFirstRepeatWaitTime
	ld	(lastKeyFirstRepeatTimer), bc
	; inverse mode: unset this
	ei	; Not done by SetUpInterrupts
MainMenu:
	ld	sp, topOfStack
	ld	a, apdEnabledMask
	ld	(mflags + appFlags), a
	xor	a
	ld	(mflags + textFlags), a
	ld	(mflags + editorFlags), a
	ld	(mflags + curFlags), a
	call	ClrScrnFull
	call	ResetScreen
	ld	hl, mMain
	jp	Menu


;====== Reboot =================================================================
RebootDevice:
	ld	a, 255
	out	(pMPgA), a
	out	(pMPgAHigh), a
	jp	4000h


;====== Suspend ================================================================
MicrosApd:
	push	af
	push	bc
	push	de
	push	hl
	call	DoSuspend
	pop	hl
	pop	de
	pop	bc
	pop	af
	ret
Suspend:
	ld	hl, MainMenu
	push	hl
DoSuspend:
	
	ret


;====== Quit ===================================================================
QuitToEOS:
#ifdef ENABLE_EOS_QUIT
	di
	; Reset screen
	xor	a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	setLcdReg(lrWinLeft, 0, 0)
	setLcdReg(lrWinRight, lcdHigh(lcdWidth-1), lcdLow(lcdHeight-1))
	setLcdReg(lrWinTop, 0, 0)
	setLcdReg(lrWinBottom, lcdHigh(lcdHeight-1), lcdLow(lcdHeight-1))
	setLcdDReg(lrEntryMode, lcdDispCtrlDefault)
	ld	sp, (spaddr)
	ld	a, (pageAOrig)
	out	(pMPgA), a
	ld	a, (pageAOrig+1)
	out	(pMPgAHigh), a
	ld	a, (pageBOrig)
	out	(pMPgB), a
	ld	a, (pageBOrig+1)
	out	(pMPgBHigh), a
	ld	hl, (quitaddr)
	jp	(hl)
#else
	; Uh ... just boot whatever OS is there, I guess
	call	ClrScrnFull
	jp	0053h
#endif


;====== Support Routines =======================================================
;------ Panic Handler ----------------------------------------------------------
Panic:
	pop	hl
	ld	sp, topOfStack
	push	hl
	call	ResetScreen
	ld	hl, strPanic
	call	PutS
	pop	hl
	call	DispHL
_:	call	RawGetCSC
	or	a
	jr	nz, -_
	jp	RebootMicrOS
	

;------ Get Version ------------------------------------------------------------
GetVersion:
	ld	hl, MicrOSVersion
	;ld	a, IS_OS_VAL + RUN_FROM_FLASH_VAL + EDITOR_8000h_VAL + ENABLE_USB_SUPPORT_VAL + IS_APP_VAL
	ret



;====== Includes ===============================================================
#include "utility.asm"

#include "textmode.asm"

#include "portmon.asm"

#include "lcdportmon.asm"

#include "hexeditor.asm"

#include "memory.asm"

#include "interrupts.asm"

#include "keyboard.asm"

#include "payload.asm"

;#ifdef ENABLE_USB_SUPPORT
;	#include "directusb.asm"
;#endif

#ifdef ENABLE_DISASM
disasm_code_start:
	#include "disassembler.asm"
disasm_code_end:
#endif

end_of_code_segment:

;#ifdef ENABLE_USB_SUPPORT
;	#include "directusb_data.asm"
;#endif

#ifdef ENABLE_DISASM
disasm_data_start:
	#include "disassembler_data.asm"
disasm_data_end:
#endif

#include "micros_data.asm"

#include "keyboard_data.asm"

#include "portmon_data.asm"

#include "lcdportmon_data.asm"

#include "hexeditor_data.asm"

#include "payload_data.asm"

#include "font_data.asm"
end_of_data_segment:

;------ Information ------------------------------------------------------------
.echo	" "
.echo	"INFORMATION & STATISTICS"
.echo	"Version word: ", convhex(MicrOSVersion)
#ifdef IS_OS
 .echo	"Number of entries in the jump table: ", (last_jump_table_entry_point - first_jump_table_entry_point) / 3
#endif
.echo	"Location of static vars: ", convhex(start_of_static_vars)
.echo	"Size of static vars: ", size_of_static_vars, " bytes"
.echo	"Last address before bookmarks: ", convhex(editBookmarks);lastCurCol)
.echo	"End of static vars: ", convhex(end_of_static_vars)
.echo	"Code segment size: ", end_of_code_segment - starting_address, " bytes"
.echo	"Data segment size: ", end_of_data_segment - end_of_code_segment, " bytes"
#ifdef ENABLE_DISASM
 .echo	"Disassembler code size: ", disasm_code_end - disasm_code_start, " bytes"
 .echo	"Disassembler data size: ", disasm_data_end - disasm_data_start, " bytes"
 .echo	"Number of instruction strings: ", last_instr_string
#endif
.echo	"Total MicrOS size: ", end_of_data_segment - starting_address, " bytes"
.echo	"Last address of executable code: ", convhex(end_of_code_segment)
.echo	"Last address used by MicrOS: ", convhex(end_of_data_segment)
.echo	" "

.end
.end
