; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

;Memory: currentRow, currentCol
;windLeft, windRight, windTop, windBottom
;tab1, tab2, tab3, tab4
;Flags: Inverse, scroll enable, scroll happened
;Functions:
;+PutC
;+ClrScrnFull
;+PutS, +ClearWind, +PutSpace, +PutSpaces, +PutSClrWind
;+HomeUp, +NewLine, +ScrollUp, +NewLineClrEOL
;+Locate, +NextTab, CursorRight, CursorLeft, CursorUp, CursorDown, +AdvanceCursor, +CursorBack
;+CursorForward
;SetWind, +SetFullScrnWind, +ClearWind
;CenterText
;DrawBox, DrawBanner, DrawBoxTitle
;+ReadChar
;WriteSideChan, ReadSideChan
;+SetDirectionDown
;+SetDirectionRight
;+SetCharRow, +SetCharCol, +SetLcdWindowColumnBounds, +FixCursor, +ResetScreen, NormalizeScreen
;+InitScreenModes
;+FillScrnFull


; To do: Fix New line, putsclrwind, make sure ScrollUp won't infinite loop on the last character of PutSpaces
; And yeah.
; Also, rework everything for double-size 5x7 characters.
; And write a test program for the double-width mode.
; And ScrollUp needs to pay attention to the high bit of the read character.


;------ ScrollUp ---------------------------------------------------------------
ScrollUp:
; Scrolls the window up and erases the bottom-most line, leaving the cursor at
; the left side of the line.
; Well, it's supposed to, anyway.  I need to figure out how to prevent this from
; having infinite recursion issues.
; Input:
;  - Character codes in LCD pixel subcode
; Output:
;  - Screen scrolled
; Destroys:
;  - Any chance of reasonable performance
;  - AF.  Why would you call this when you need to save AF?
;  - B
	
	ret
	
	push	hl
	ld	a, (windBottom)
	ld	h, a
	ld	a, (windTop)
	sub	h
	jr	c, scrollUpSpaces
	jr	z, scrollUpSpaces
	ld	a, (windRight)
	ld	(currentCol), a
	ld	a, (windTop)
	ld	l, a
_:	ld	a, (currentCol)
	ld	h, a
	inc	l
	call	Locate
	call	ReadChar
	dec	l
	call	Locate
	call	PutC
	ld	a, (windRight)
	cp	h
	jr	nz, -_
	ld	a, (windRight)
	ld	(currentCol), a
	inc	l
	ld	a, (windBottom)
	cp	l
	jr	nz, -_
scrollUpSpaces:
	ld	a, (windLeft)
	ld	h, a
	ld	a, (windRight)
	sub	h
	inc	a
	ld	b, a
	call	PutSpaces
	ld	a, (windLeft)
	ld	h, a
	ld	a, (windBottom)
	ld	a, l
	call	Locate
	pop	hl
	ret


	


;------ SetFullScrnWind --------------------------------------------------------
SetFullScrnWind:
; Resets the window to one in which everything is full-screen
; Inputs:
;  - None
; Output:
;  - Window set
; Destroys:
;  - BC
	ld	bc, 0
	ld	(windTop), bc
	ld	bc, (textCols) * 256 + textRows
	ld	(windBottom), bc
	ret


;------ ClearWind --------------------------------------------------------------
ClearWind:
; Clears the current window by printing lots of spaces.
; Input:
;  - Window bounds
; Output:
;  - Window cleared and cursor moved to top left
; Destroys:
;  - Assume everything
	call	HomeUp
_:	call	NewLineClrEOL
	ld	a, (windTop)
	ld	b, a
	ld	a, (currentRow)
	cp	b
	jr	nz, -_
	jp	HomeUp


;------ PutSClrWind ------------------------------------------------------------
PutSClrWind:
; Displays a string.  If the string contains control codes, those codes are
; parsed.  New lines will print spaces until the end of the line.
; Input:
;  - HL: String to show
; Output:
;  - String shown
;  - HL advanced to the byte after the null terminator.
; Destroys:
;  - AF
;  - HL
	push	bc
	push	de
putSClrWindLoop:
	ld	a, (hl)
	inc	hl
	cp	ch1stPrintableChar
	call	nc, PutC2
	jr	nc, putSClrWindLoop
	or	a
	jr	z, putSClrWindFillToEnd
	cp	chNewLine
	jr	nz, +_
	call	NewLineClrEOL
	jr	putSClrWindLoop
_:	cp	chTab
	call	z, NextTab
	jr	putSClrWindLoop	; Just display spaces until the last column of the last line
putSClrWindFillToEnd:
	push	hl
	ld	a, (windTop)
	ld	b, a
	ld	a, (currentRow)
	cp	b
	jr	nz, +_
	ld	a, (windLeft)
	ld	b, a
	ld	a, (currentCol)
	cp	b
	jr	z, putSClrWindRet
_:	call	NewLineClrEOL
	ld	a, (windTop)
	ld	b, a
	ld	a, (currentRow)
	cp	b
	jr	nz, -_
putSClrWindRet:
	call	HomeUp
	pop	hl
	pop	de
	pop	bc
	ret


;------ PutS -------------------------------------------------------------------
PutS:
; Displays a string.  If the string contains control codes, those codes are
; parsed.
; Input:
;  - HL: String to show
; Output:
;  - String shown
;  - HL advanced to the byte after the null terminator.
; Destroys:
;  - AF
;  - HL
	ld	a, (hl)
	inc	hl
	cp	ch1stPrintableChar
	call	nc, PutC2
	jr	nc, PutS
	or	a
	ret	z
	push	af
	push	bc
	push	de
	push	hl
	push	ix
	cp	chNewLine
	jr	nz, +_
	call	NewLine
	jr	++_
_:	cp	chTab
	call	z, NextTab
_:	pop	ix
	pop	hl
	pop	de
	pop	bc
	pop	af
	jr	PutS


;------ Locate -----------------------------------------------------------------
Locate:
; Moves the cursor to a specific location.
; Inputs:
;  - H: Column
;  - L: Row
; Output:
;  - Cursor moved
; Destroys:
;  - Nothing
	ld	(currentRow), hl
	push	af
	push	bc
	push	de
	push	hl
	call	SetCharRow
	call	SetCharCol
	pop	hl
	pop	de
	pop	bc
	pop	af
	ret


;------ HomeUp -----------------------------------------------------------------
HomeUp:
; Moves the text cursor up to the top, left of the text window.
; Inputs:
;  - None
; Output:
;  - Cursor moved
; Destroys:
;  - Everything, or maybe nothing
	ld	a, (windTop)
	ld	(currentRow), a
	ld	a, (windLeft)
	ld	(currentCol), a
	call	SetCharRow
	jp	SetCharCol

	
;------ CursorBack -------------------------------------------------------------
CursorBack:
; Moves the cursor back one unit.  May move to the rightmost position of the
; previous line.  May also move to bottom-right of window if cursor was
; previously at top-left.
; Input:
;  - Current cursor location
; Output:
;  - Cursor moved.
; Destroys:
;  - AF
;  - C
	ld	a, (currentCol)
	or	a
	jr	+_
	ld	c, a
	ld	a, (windLeft)
	cp	c
	jr	z, +_
	ld	a, c
	dec	a
	ld	(currentCol), a
	jp	SetCharCol
_:	ld	a, (windRight)
	ld	(currentCol), a
	ld	a, (currentRow)
	or	a
	ret	z
	ld	c, a
	ld	a, (windTop)
	cp	c
	;ret	z
	jr	nz, +_
	ld	a, (windBottom)
	ld	c, a
_:	ld	a, c
	dec	a
	ld	(currentRow), a
	jp	FixCursor


;------ CursorForward ----------------------------------------------------------
CursorForward:
; Moves the cursor forward one unit.
; Input:
;  - Current cursor location
; Output:
;  - Cursor moved.
; Destroys:
;  - Whatever AdvanceCursor destorys.
;  - AF
	call	AdvanceCursor
	jp	FixCursor


;------ CursorRight ------------------------------------------------------------
CursorRight:
; Moves the cursor right, but does not allow the cursor to move past the right-
; most edge of the window.
; Input:
;  - Current cursor location
; Output:
;  - Cursor moved.
; Destroys:
;  - AF
;  - C
;  - HL
	ld	a, (windRight)
	ld	c, a
	ld	a, (currentCol)
	inc	a
	cp	c
	ret	nc
	ld	(currentCol), a
	jp	SetCharCol


;------ CursorLeft -------------------------------------------------------------
CursorLeft:
; Moves the cursor left, but does not allow the cursor to move past the left-
; most edge of the window.
; Input:
;  - Current cursor location
; Output:
;  - Cursor moved.
; Destroys:
;  - AF
;  - C
;  - HL
	ld	a, (currentCol)
	dec	a
	ld	c, a
	ld	a, (windLeft)
	cp	c
	ret	nc
	ld	(currentCol), a
	jp	SetCharCol


;------ CursorDown -------------------------------------------------------------
CursorDown:
; Moves the cursor down one row, but won't move down past the bottom of the
; window.
; Input:
;  - Current cursor location
; Output:
;  - Cursor moved.
; Destroys:
;  - AF
;  - BC
;  - E
	ld	a, (windBottom)
	ld	c, a
	ld	a, (currentRow)
	inc	a
	cp	c
	ret	nc
	ld	(currentRow), a
	jp	SetCharRow
	
;------ CursorUp ---------------------------------------------------------------
CursorUp:
; Moves the cursor upone row, but won't move down past the top of the window.
; Input:
;  - Current cursor location
; Output:
;  - Cursor moved.
; Destroys:
;  - AF
;  - BC
;  - E
	ld	a, (currentRow)
	dec	a
	ld	c, a
	ld	a, (windTop)
	cp	c
	ret	nc
	ld	(currentRow), a
	jp	SetCharRow


;------ AdvanceCursor ----------------------------------------------------------
AdvanceCursor:
; After a function like PutSpace or PutC has draw the character, this updates
; the text cursor to match.  This may trigger advancing to a new line.
; It may also trigger scrolling, which will really slow things down.
; Input:
;  - textAutoScroll, (iy + textFlags)
; Output:
;  - Cursor advanced.
; Destroys:
;  - A
;  - B
;  - If scrolling is triggered, throw in C, DE, HL, and IX.
	ld	a, (currentCol)
	inc	a
	ld	(currentCol), a
	ld	b, a
	ld	a, (windRight)
	cp	b
	jr	z, NewLine
	ret	nc


;------ NewLine ----------------------------------------------------------------
; Advances the cursor to the next line.
; Inputs:
;  - None
; Output:
;  - Cursor moved.
; Destroys:
;  - A
;  - B
;  - If scrolling is triggered, throw in C, DE, HL, and IX.
NewLine:
	;ld	a, (mflags + textFlags)
	;bit	textNewlineClr, a
	;jp	nz, NewLineClrEOL
	ld	a, (windLeft)
	ld	(currentCol), a
	ld	a, (currentRow)
	inc	a
	ld	(currentRow), a
	ld	b, a
	ld	a, (windBottom)
	cp	b
	;jr	nc, +_
	jr	z, +_
	jp	nc, FixCursor
_:	ld	a, (windTop)
	ld	(currentRow), a
;	ld	a, (mflags + textFlags)
;	bit	textAutoScroll, a
;	jp	nz, ScrollUp
	jp	FixCursor


;------ NewLineClrEOL ----------------------------------------------------------
NewLineClrEOL:
; Moves the cursor to the next line, clearing the remainder of the current line.
; Inputs:
;  - None
; Output:
;  - Cursor moved
; Destroys:
;  - Everything, or maybe nothing
	call	PutSpace
;	ld	a, 05Fh
;	call	PutC
	ld	a, (currentCol)
	ld	b, a
	ld	a, (windLeft)
	cp	b
	jr	nz, NewLineClrEOL
	ret


;------ NewLine2 ---------------------------------------------------------------
NewLine2:
; Moves the cursor to the next line, unless it is already at the start of a new
; line.  Useful if you want to display data all the way up to the last column.
	ld	a, (currentCol)
	ld	b, a
	ld	a, (windLeft)
	cp	b
	jp	nz, NewLine
	ret


;------ NewLineClrEOL2 ---------------------------------------------------------
NewLineClrEOL2:
; Moves the cursor to the next line, clearing the remainder of the current line,
; unless the cursor is already at the start of a new line.  
; Useful if you want to display data all the way up to the last column.
; Inputs:
;  - None
; Output:
;  - Cursor moved
; Destroys:
;  - Everything, or maybe nothing
	ld	a, (currentCol)
	ld	b, a
	ld	a, (windLeft)
	cp	b
	jp	nz, NewLineClrEOL
	ret


;------ NextTab ----------------------------------------------------------------
NextTab:
; Moves the cursor forward to the next tab position.
; Inputs:
;  - Tab array
;  - currentCol
; Output:
;  - Cursor advanced
; Destroys:
;  - Nothing
	push	af
	push	bc
	push	de
	push	hl
	ld	hl, currentCol
	ld	a, (hl)
	inc	hl
	ld	b, (hl)
	inc	hl
	cp	b
	jr	c, +_
	ld	b, (hl)
	inc	hl
	cp	b
	jr	c, +_
	ld	b, (hl)
	inc	hl
	cp	b
	jr	c, +_
	ld	b, (hl)
	inc	hl
	cp	b
	jr	nc, tabRestoreRegisters
_:	ld	c, a
	ld	a, b
	sub	c
	ld	b, a
	call	PutSpaces
tabRestoreRegisters:
	pop	hl
	pop	de
	pop	bc
	pop	af
	ret


;------ PutSpace ---------------------------------------------------------------
PutSpace:
; Draws a single blank cell to the screen.
; Input:
;  - textInverse, (iy + textFlags)
; Output:
;  - Spaces written
; Destroys:
;  - A
;  - B
; Assumes:
;  - LCD window already set
;  - LCD write cursor already set
	; Tell it we want to write pixels
	ld	a, lrGram
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	ld	b, (charWidth * charHeight) / 2
	ld	a, (mflags + textFlags)
	bit	textInverse, a
	ld	a, 0	; Do not change to XOR A
	jr	z, +_
	dec	a
_:	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	djnz	-_
	jp	AdvanceCursor


;------ PutSpaces --------------------------------------------------------------
PutSpaces:
; Sends several spaces to the screen.  This routine may cause scrolling if
; scrolling is enabled.
; Input:
;  - B: Number of space to display
; Output:
;  - Cursor moved, space erased.
; Destroys:
;  - AF
;  - B
;  - DE
;  - IX if the screen is scrolled
	push	bc
	push	hl
	call	PutSpace
	pop	hl
	pop	bc
	djnz	PutSpaces
	ret


;------ PutC2 ------------------------------------------------------------------
PutC2:
; Displays the character given, just like PutC.  However, this checks bit 7 to
; see whether the character should be displayed in inverse mode or not.
; If inverse mode is set in textFlags, then the meaning of bit 7 is reversed;
; that is, it inverts the inversion.
; Inputs:
;  - A: Character
;  - 7,A: Set for inverse mode.
	push	af
	push	bc
	ld	b, a
	ld	a, (mflags + textFlags)
	push	af
	ld	c, a
	and	textInverseMask^255
	bit	textInverse, c
	ld	a, 0
	jr	z, +_
	xor	textInverseMask
_:	bit	7, b
	jr	z, +_
	xor	textInverseMask
	res	7, b
_:	or	c
	ld	(mflags + textFlags), a
	ld	a, b
	call	PutC	
	pop	af
	ld	(mflags + textFlags), a
	pop	bc
	pop	af
	ret


;------ PutC -------------------------------------------------------------------
PutC:
; Draws a character to the screen.
; Input:
;  - A
; Output:
;  - Character drawn
;  - Cursor advanced to next location
; Destroys:
;  - Nothing
; Assumes:
;  - LCD window already set
;  - LCD write cursor already set
	push	af
	push	bc
	push	de
	push	hl
	push	ix
	ld	ix, mflags
	ld	c, a
	xor	a
	; Sync
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	; Tell it we want to write pixels
	ld	a, lrGram
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	ld	a, c
;	sub	minChar	; Not all 256 code points have bitmaps
;	jr	nc, +_
;	xor	a
;_:	; Multiply by 16
	ld	h, 0
	ld	l, a
	add	hl, hl
	add	hl, hl
	add	hl, hl
	add	hl, hl
	ld	de, fontData
	add	hl, de
; Inital output: Write ASCII code
	; Set bit 7 in ASCII if inverse text is set
	bit	textInverse, (ix+textFlags)
	jr	z, +_
	or	80h
	ld	c, a
_:	ld	a, (hl)
	inc	hl
	ld	b, 8
	bit	textInverse, (ix+textFlags)
	jr	z, +_
	cpl
_:	ld	e, a
_:	; Output byte loop
	rrc	e
	sbc	a, a
	out	(pLcdData), a
	rrc	c
	jr	nc, +_
	xor	20h
_:	out	(pLcdData), a
	djnz	--_
; Main output loop
	ld	c, charLength-2
PutMapOutLoop:
	ld	a, (hl)
	inc	hl
	ld	b, 8
	bit	textInverse, (ix+textFlags)
	jr	z, +_
	cpl
_:	ld	e, a
_:	; Output byte loop
	rrc	e
	sbc	a, a
	out	(pLcdData), a
	out	(pLcdData), a
	djnz	-_
	dec	c
	jr	nz, PutMapOutLoop
; Remainder
	ld	a, (hl)
	ld	b, 6
	bit	textInverse, (ix+textFlags)
	jr	z, +_
	cpl
_:	ld	e, a
_:	; Output byte loop
	rrc	e
	sbc	a, a
	out	(pLcdData), a
	out	(pLcdData), a
	djnz	-_
	call	AdvanceCursor
	pop	ix
	pop	hl
	pop	de
	pop	bc
	pop	af
	ret


;------ ReadChar ---------------------------------------------------------------
; Reads the character code of the character currently under the cursor.
; WARNING: The LCD cursor is not reset!
; To advance to next character location, call SetCharCol AND SetCharRow to reset
; to previous value.  If you want to advance to the next location, call
; AdvanceCursor with textAutoScroll disabled.  (If it's enabled, calling
; AdvanceCursor could cause scrolling, which is probably not what you want since
; you're trying to READ non-destructively.)
; Inputs:
;  - Set LCD cursor to the correct location
; Output:
;  - A: 8-bit character code
; Destroys:
;  - LCD cursor
ReadChar:
	push	bc
	push	de
	push	hl
	ld	c, pLcdData
	ld	b, 8
	ld	a, lrGram
	out	(pLcdCmd), a
	out	(pLcdCmd), a
_:	in	a, (pLcdData)
	in	a, (pLcdData)
	in	h, (c)
	in	a, (pLcdData)
	ld	l, a
	xor	h
	add	a, a
	add	a, a
	add	a, a
	rr	e
	out	(c), h
	out	(c), l
	djnz	-_
	ld	a, e
	pop	hl
	pop	de
	pop	bc
	ret


;------ ResetScreen ------------------------------------------------------------
ResetScreen:
; Initalizes the screen modes and text control variables.  This must be the
; first thing you call for text mode.
; Inputs:
;  - None
; Output:
;  - Everything ready for text mode
; Destroys:
;  - I dunno, assume everything.
	xor	a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	call	SetDirectionDown
	call	SetLcdWindowColumnBounds
	call	SetFullScrnWind
	jp	HomeUp


;------ FixCursor --------------------------------------------------------------
FixCursor:
; Fixes up the LCD read/write cursor
; Inputs:
;  - None
; Output:
;  - LCD window set
; Destroys:
;  - AF
	push	bc
	push	de
	push	hl
	call	SetCharCol
	call	SetCharRow
	pop	hl
	pop	de
	pop	bc
	ret


;------ SetCharRow -------------------------------------------------------------
SetCharRow:
; Sets the LCD window top and bottom for the current row of text.
; Inputs:
;  - None
; Output:
;  - LCD window set
; Destroys:
;  - AF
;  - BC
;  - E
	; Setup
	ld	b, lrWinTop
	ld	c, pLcdCmd
	; Set row
	ld	a, (currentRow)
	; Multiply by 14
	add	a, a
	ld	e, a
	add	a, a
	add	a, a
	add	a, a
	sub	e
	; And write it
	out	(c), b
	out	(c), b
	inc	c
	out	(c), 0
	out	(pLcdData), a
	dec	c
	; lrWinRowEnd
	inc	b
	out	(c), b
	out	(c), b
	add	a, charHeight-1
	inc	c
	out	(c), 0
	out	(pLcdData), a
	dec	c
	; lrRow
	ld	b, lrRow
	out	(c), b
	out	(c), b
	sub	charHeight-1
	inc	c
	out	(c), 0
	out	(pLcdData), a
	dec	c
	ret


;------ SetCharCol -------------------------------------------------------------
SetCharCol:
; Sets the LCD write cursor column to match currentCol
; Input:
;  - None
; Output:
;  - LCD window set
; Destroys:
;  - AF
;  - C
;  - HL
	; Set column
	ld	a, (currentCol)
	; Multiply by 9
	ld	h, a
	add	a, a
	add	a, a
	add	a, a
	add	a, h
	ld	l, a
	ld	a, h
	cp	29
	sbc	a, a
	inc	a
	ld	h, a
	ld	a, lrCol
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	ld	a, h
	out	(pLcdData), a
	ld	a, l
	out	(pLcdData), a
;	dec	c
	; lrWinColStart
;	ld	h, a
;	ld	de, charWidth
;	add	hl, de
	; lrWinColEnd
;	inc	b
;	out	(c), 0
;	out	(c), b
;	inc	c
;	out	(c), h
;	out	(c), l
	ret


;------ SetLcdWindowColumnBounds -----------------------------------------------
; Sets the LCD window left/right bounds.
; These should be 0 and 314, because MicrOS rarely needs to keep writing to the
; same char location.
; Input:
;  - None
; Output:
;  - LCD left/right bounds reset
; Destroys:
;  - A
; Assumes:
;  - Assumes that you want to be text mode.
; Execution Time:
;  - 134 cc not including inital CALL
SetLcdWindowColumnBounds:
	xor	a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	ld	a, lrWinLeft
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	xor	a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdCmd), a
	ld	a, lrWinRight
	out	(pLcdCmd), a
	ld	a, 1
	out	(pLcdData), a
	ld	a, lcdLow(((charWidth * textCols) - 1))
	out	(pLcdData), a
	ld	a, lrWinBottom
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	xor	a
	out	(pLcdData), a
	ld	a, colorScrnHeight
	out	(pLcdData), a
	ret	;134 cc


;------ SetDirectionRight ------------------------------------------------------
SetDirectionRight:
; Sets the cursor to move left after every pixel.  At the end of the row, it
; then moves down;
; Inputs:
;  - None
; Output:
;  - Mode changed
; Destroys:
;  - AF
;  - H
	push	hl
	ld	a, lrEntryMode
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	in	a, (pLcdData)
	ld	h, a
	in	a, (pLcdData)
	; Ignore the result
	ld	a, h
	out	(pLcdData), a
	ld	a, lcdCurMoveHoriz | lcdRowInc | lcdColInc
	out	(pLcdData), a
	pop	hl
	ret


;------ SetDirectionDown -------------------------------------------------------
SetDirectionDown:
; Sets the cursor to move down after every pixel.  At the end of the column, it
; then moves right;
; Inputs:
;  - None
; Output:
;  - Mode changed
; Destroys:
;  - AF
;  - H
	ld	a, lrEntryMode
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	in	a, (pLcdData)
	ld	h, a
	in	a, (pLcdData)
	; Ignore the result
	ld	a, h
	out	(pLcdData), a
	ld	a, lcdRowInc | lcdColInc
	out	(pLcdData), a
	ret


;------ ClearSrcnFull ----------------------------------------------------------
; Fully erases the whole screen to black.
ClrScrnFull:
	ld	d, 0
;------ FillScrnFull -----------------------------------------------------------
FillScrnFull:
; Fills the entire screen with a specified 16-bit color.
; Selected colors:
; Component 5/6/5 is the color as the LCD controller sees it.  For consistency,
; the middle green channel is divided by two, so each channel is out of 31.
; Component 8/8/8 is each channel multiplied by 8 so it is out of 255, like on
; PCs.
; Hex   Component 5/6/5	Component 8/8/8	Color RGB	Color BGR
; 00	 0,    0,  0	  0,   0,   0	Black		Black
; 06	 0,   24,  6	  0, 192,  48	Green		Green
; 08	 1,    0,  8	  8,   0,  64	Navy Blue	Dark Maroon?
; 18	 3,    0, 24	 24,   0, 192	Blue		Red
; 1F	 3,   28, 31	 24, 224, 248	Baby Blue?	Gold/Yellow?
; 4A	 9,    9, 10
; 60	12,  1.5,  0	 96,  12,   0	Maroon?		Dark Blue?
; 33	 0,   12,  3	  0,  96,  24	Dark Green	Better Dark Green
; 6B	13, 13.5, 11	104, 108,  88	Gray 39%	Gray 39%
;;; 77	28, 31.5,  7	224, 252,  56	Lime Green	Teal
; 77	 0,   28,  7	  0, 224,  26			Green-screen Green
; B5	22, 22.5, 21	176, 180, 168	Gray 68%	Gray 68%
; D8	27,    3, 24	216,  24, 192	Magenta?	Blue-Magenta? 
; DB	27,   27, 30	216, 216, 240	Gray 88%	Gray 88%
; E0	28,  3.5,  0	224,  28,   0	Red		Blue
; FF	31,   31, 31	255, 255, 255	White		White
; 
; Input:
;  - D: Color to write, high and low bytes are the same
; Output:
;  - Screen filled with repeating byte
;  - Current LCD register set to lrGram
;  - Row & Col set to (0,0)
;  - Window set to (0,0)-(319,239)
; Destroys:
;  - A, BC
; Assumes:
;  - Only assumes that the LCD is more or less configured correctly.
; Execution Time:
;  - Base time is 129 ms
;  - Add 31 ms if ASIC-forced LCD delay is set to the minimum of 3 clock cycle
;  - Total minimum time is therefore 160 ms (or 6.2 frames/second)
	ld	c, pLcdCmd	; 7 cc
	xor	a		
	;			; sigma = 11
	; Sync
	out	(pLcdCmd), a	; 11 cc
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	;			; sigma = 44
	; lrWinTop
	ld	b, lrWinTop	; 7 cc
	out	(pLcdCmd), a
	out	(c), b		; 12 cc
	out	(pLcdData), a
	out	(pLcdData), a
	;			; sigma = 52
	; lrWinBottom
	inc	b
	out	(pLcdCmd), a
	out	(c), b
	out	(pLcdData), a
	ld	a, colorScrnHeight-1
	out	(pLcdData), a
	xor	a
	;			; sigma = 60
	; lrWinLeft
	inc	b
	out	(pLcdCmd), a
	out	(c), b
	out	(pLcdData), a
	out	(pLcdData), a
	;			; sigma = 49
	; lrWinRight
	inc	b
	out	(pLcdCmd), a
	out	(c), b
	inc	a
	;ld	de, colorScrnWidth ; 10 cc
	out	(pLcdData), a
	ld	a, lcdLow((colorScrnWidth-1))
	out	(pLcdData), a
	xor	a
	;			; sigma = 64
	; lrRow
	ld	b, lrRow
	out	(pLcdCmd), a
	out	(c), b
	out	(pLcdData), a
	out	(pLcdData), a
	;			; sigma = 52
	; lrCol
	inc	b
	out	(pLcdCmd), a
	out	(c), b
	out	(pLcdData), a
	out	(pLcdData), a
	;			; sigma = 49
	; Now fill the screen
	ld	b, lrGram
	out	(pLcdCmd), a
	out	(c), b
	ld	c, 75
	ld	b, 0
	ld	a, d
	;			; sigma = 51
	; Total time for header:  sigma = 432
	; Plus 3 cycles added by ASIC per write: 30*3 = 90
	; So 522 cc total for initalization.
_:	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	djnz	-_		; 13/8 cc
	; 11*8*256+13*255+8 = 25851 cc
	dec	c
	jr	nz, -_		; 12/7 cc
	; (25851+4)*75+12*74+7 = 1940020
	ret			; 10 cc
	; 1940462 = ca. 129 ms
	; Plus 3 cycles per write: 3*2*320*240 = 460800.
	; So, including delays, 2401352 clock cycles.
	; Which is about 160 ms or 6.2 frames/second.