; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.


;====== Math Routines ==========================================================
;------ Milos Bazelides routines -----------------------------------------------
; These routines are by Milos "baze" Bazelides, baze_at_baze_au_com
; Retrived from http://baze.au.com/misc/z80bits.html 25 October 2008

MultHByE:
; Input: H = Multiplier, E = Multiplicand
; Output: HL = Product
	xor	a
	ld	l, a
	ld	d, a
	sla	h		; optimised 1st iteration
	jr	nc,$+3
	ld	l,e
	
	ld	b, 7
multHbyEloop:
	add	hl,hl		; unroll 7 times
	jr	nc,$+3		; ...
	add	hl,de		; ...
	djnz	multHbyEloop
	ret

;------ ------------------------------------------------------------------------
DispHL:
; Input:
;  - HL
; Output:
;  - Word displayed
; Destorys:
;  - AF
	ld	a, h
	call	DispByte
	ld	a, l
DispByte:
; Input:
;  - A: Byte
; Output:
;  - Byte displayed
; Destorys:
;  - AF
	push	af
	rra
	rra
	rra
	rra
	call	+_
	pop	af
_:	or	0F0h
	daa
	add	a, 0A0h
	adc	a, 40h
	call	PutC
	ret


;------ ------------------------------------------------------------------------
DispDecimal:
; Input:
;  - HL
; Output:
;  - 16-bit number displayed in decimal
; Destorys:
;  - BC
;  - AF
	ld	bc,-10000
	call	+_
	ld	bc,-1000
	call	+_
	ld	bc,-100
	call	+_
	ld	c,-10
	call	+_
	ld	c,b
_:	ld	a,'0'-1
_:	inc	a
	add	hl,bc
	jr	c, -_
	sbc	hl,bc
	call	PutC
	ret


;====== Routines ===============================================================

;------ ------------------------------------------------------------------------
ClearMem:
; Clears a section of memory SLOW! :p
; Input:
; - HL: Location to kill
; - B: Number of bytes to kill
; - A: What to clear it with
	ld	(hl), a
	inc	hl
	djnz	ClearMem
	ret


;------ Process Key Jump Table -------------------------------------------------
ProcessKeyJumpTable:
; Inputs:
;  - HL: Table
;  - A: Keycode
	ld	c, a
_:	ld	a, (hl)
	inc	hl
	or	a
	ret	z
	cp	c
	jr	z, +_
	inc	hl
	inc	hl
	jr	-_
_:	ld	a, (hl)
	inc	hl
	ld	h, (hl)
	ld	l, a
	pop	af
	xor	a
	jp	(hl)


;------ Menu -------------------------------------------------------------------
Menu:
; Simple menu routine.
; HL points to a data structure that describes the menu.
; The first item in the menu is a zero-termed string to be displayed.
; After that, there is a list of acceptable keys and jump addresses.
; This is a JUMP not a CALL!
; This passes the key that was pressed in B.
	di
	ld	a, (mflags + appFlags)
	and	textAutoScrollMask ^ 255
	ld	(mflags + appFlags), a
	push	hl
;	call	ClearWind
	call	HomeUp
	pop	hl
	call	PutSClrWind ; hl = byte after 0
	ld	e, l
	ld	d, h
menuKeyLoop:
;	push	de
	call	GetKey	; Should preserve all in current MicrOS
;	pop	de
	ld	l, e
	ld	h, d
	ld	b, a
	ld	a, (hl) ; sanity check
	or	a ;
	jp	z, MainMenu
_:	; Key table scan loop
	ld	a, (hl)
	or	a
	jr	z, menuKeyLoop
	inc	hl
	cp	b
	jr	z, +_
	inc	hl
	inc	hl
	jr	-_
_:	; Found an entry!  Is it a link to another menu?
	ldHlIndirect
	ld	a, (hl)
	or	a
	jr	z, +_
	jp	(hl)
_:	; Yes, it's linking to another menu.
	inc	hl
	jp	Menu


;------ AnyKey -----------------------------------------------------------------
AnyKey:
; Waits until a key is pressed, then returns
; Inputs:
;  - None
; Output:
;  - Waits until a key is pressed, then returns
; Destroys:
;  - AF
;  - BC
	ld	bc, 0
	ld	a, 0FFh
	out	(pKey), a
	djnz	$
	inc	a
	out	(pKey), a
	djnz	$
_:	in	a, (pKey)
	inc	a
	jr	z, -_
	ld	a, 0FFh
	out	(pKey), a
_:	djnz	$
	dec	c
	jr	nz, -_
	ret